package com.jesusrojo.miphoto.model;


import android.content.Context;

import android.graphics.Bitmap;
import android.net.Uri;
import android.text.TextUtils;


import com.jesusrojo.miphoto.model.almacen.PhotosWebService;
import com.jesusrojo.miphoto.model.almacen.IAlmacen;
import com.jesusrojo.miphoto.model.almacen.UsersSQLite;
import com.jesusrojo.miphoto.model.pojo.Photo;
import com.jesusrojo.miphoto.model.pojo.User;
import com.jesusrojo.miphoto.utils.Constants;
import com.jesusrojo.miphoto.utils.Utils;
import com.jesusrojo.miphoto.utils.UtilsFile;
import com.jesusrojo.miphoto.utils.UtilsFilter;
import com.jesusrojo.miphoto.view.PhotosListActivity;


import java.io.File;
import java.util.List;

/**
 * Mediates communication between the  Service and the local
 * storage on the Android device.  The methods in this class block, so
 * they should be called from a background thread (e.g., via an
 * AsyncTask).
 */
public class Mediator {
    private final String TAG = getClass().getSimpleName();
    private Context mContext;

    ////
    public Mediator() { }
    public Mediator(Context context) {
        this.mContext = context;
    }

    public String register(String name, String password, String mail, String fromWhere) {
        if (TextUtils.isEmpty(name)
                || TextUtils.isEmpty(password)
                || TextUtils.isEmpty(mail) ){
            return Constants.ERROR;

        } else if (fromWhere.equals("device")) {

            UsersSQLite usersSQLite = new UsersSQLite(mContext);
            User user =  new User(-1, name, password,mail, System.currentTimeMillis());
            return usersSQLite.registerUser(mContext, user);

        } else if (fromWhere.equals("cloud")) {

            IAlmacen webService = new PhotosWebService( mContext );
            return webService.register(name, password, mail);

        } else {
            return Constants.ERROR;
        }
    }
    public String checkLogin(Context c, String name, String password, String fromWhere) {
        if (TextUtils.isEmpty(name)
                || TextUtils.isEmpty(password) ){
            return Constants.ERROR;

        } else if (fromWhere.equals("device")) {

            UsersSQLite usersSQLite = new UsersSQLite(mContext);
            return usersSQLite.checkLogin(name, password);

        } else if (fromWhere.equals("cloud")) {

            IAlmacen webService = new PhotosWebService( c );
            return webService.checkLogin(name, password);

        } else {
            return Constants.ERROR;
        }
    }

    public String[] sendNewPassword(Context c,String mail, String fromWhere) {
        if (TextUtils.isEmpty(mail) ){
            return null;

        } else if (fromWhere.equals("device")) {

            UsersSQLite usersSQLite = new UsersSQLite(mContext);
            return usersSQLite.sendNewPassword(mail);

        } else if (fromWhere.equals("cloud")) {

            IAlmacen webService = new PhotosWebService( c );
            return webService.sendNewPassword(mail);
        } else {
            return null;
        }
    }
    public List<Photo> getListPhotos(Context c) {

        return PhotosListActivity.mIAlmacen.getListPhotos(c);
    }
    public String savePhoto(Context mContext, Photo photo) {
        return PhotosListActivity.mIAlmacen.savePhoto(mContext, photo);    }

    public String savePojoNube(Context context, Photo photo) {
        IAlmacen webService = new PhotosWebService(context);
        return webService.savePhoto(mContext, photo);
    }
   //UPLOAD 2
    public String uploadPhoto(Context context, Uri uri) {
            //subimos la binaria a la nube
            IAlmacen iAlmacen= new PhotosWebService(mContext);
           return iAlmacen.uploadBinary(mContext, uri);
    }
    public String updatePhoto(Context context, Photo photo) {

        return PhotosListActivity.mIAlmacen.updatePhoto(context, photo);
    }
    public String deletePojo(Context context, String[] selectPath) {

        return PhotosListActivity.mIAlmacen.deletePhotosByPath(selectPath);
    }
    public String deleteBinaryPhoto(Context context, Uri uri) {

       return PhotosListActivity.mIAlmacen.deleteBinary(context, uri);
    }

    public String copyBinary(Context context, Uri uri, File file) {
        return UtilsFile.copyInputStreamToMyFile(context, uri, file);
    }

    //FILTER
    public Bitmap getFilteredBitmap(Context context, Uri uri) {

        String path = String.valueOf(uri).substring(8);
        Bitmap originalImage =
                UtilsFile.decodeSampledBitmapFromFile(mContext,path, 1000, 700);

        Bitmap filteredBitmap = UtilsFilter.doGreyFilter(originalImage, Uri.parse(path));
        return filteredBitmap;
    }
    public String updatePathFilteredPhoto(Context context,String[] selectArgs) {
        return PhotosListActivity.mIAlmacen.updatePathFilter(selectArgs);
    }
    public String updateStatus(Context context, String[] selectArgs) {
        return PhotosListActivity.mIAlmacen.updateStatus(selectArgs);
    }


    ////////////////////////////////////////////////////////////////
    public Boolean deleteListPhotos(List<Photo> list) {

        if (list != null) {
                String[] selectArg = new String[list.size()];

                for (int i = 1; i < list.size(); i++) {
                    selectArg[i] = list.get(i).getPath();
                }

                if (selectArg.length != 0 && selectArg[0] != null) {

                    PhotosListActivity.mIAlmacen.deletePhotosByPath(selectArg);
                    return true;
                } else {
                    Utils.showToast(mContext, "Select photo(s)");
                    return false;
                }
            } else {
                return false;
            }
        }

}

