package com.jesusrojo.miphoto.model.almacen;

import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.net.Uri;


import com.jesusrojo.miphoto.model.pojo.Photo;
import com.jesusrojo.miphoto.utils.Constants;
import com.jesusrojo.miphoto.utils.Utils;
import com.jesusrojo.miphoto.utils.UtilsFile;

import java.util.ArrayList;
import java.util.List;

public class PhotosSQLite extends SQLiteOpenHelper  implements IAlmacen      {

    private final String TAG = getClass().getSimpleName();
    private Context mContext;
    private static final String TABLE_NAME = "table_";
    private  String mUserTableName;
    private  String mTableFullName;

    private static final String _ID= "_id";
    private static final String OWNER= "owner";
    private static final String PATH = "path";
    private static final String PATH_FILTER = "pathFilter";
    private static final String TIME = "time";

    ////
    public PhotosSQLite(Context context, String userName) {
        super(context, TABLE_NAME + userName, null, 1);
        this.mContext = context;
        this.mUserTableName = userName;
        mTableFullName = TABLE_NAME + mUserTableName;
    }

    //Métodos de SQLiteOpenHelper
    @Override
    public void onCreate(SQLiteDatabase db) {
        db.execSQL("CREATE TABLE " + mTableFullName + " (" +
                "_id INTEGER PRIMARY KEY AUTOINCREMENT" +
                ", id INTEGER" +
                ", owner TEXT" +
                ", path TEXT" +
                ", pathFilter TEXT" +
                ", latitude DOUBLE" +
                ", longitude DOUBLE" +
                ", comment TEXT" +
                ", status INTEGER" +
                ", time LONG)");
    }
    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        // En caso de una nueva versión habría que actualizar las tablas
    }

    @Override
    public String register(String name, String password, String mail) {
        //lo reenviamos a la nube
        IAlmacen iAlmacen= new PhotosWebService(mContext);
        return iAlmacen.register(name, password, mail);
    }
    @Override
    public String checkLogin(String name, String password) {
        //lo reenviamos a la nube
        IAlmacen iAlmacen= new PhotosWebService(mContext);
        return iAlmacen.checkLogin(name, password);
    }

    @Override
    public String[] sendNewPassword(String mail) {
        return null;
    }

    @Override
    public String savePhoto(Context c, Photo photo) {
        SQLiteDatabase db = getWritableDatabase();
        db.execSQL("INSERT INTO " + mTableFullName + " VALUES ( null"
                + ", " + photo.getId()
                + ", '" + photo.getOwner() + "' "
                + ", '" + photo.getPath() + "' "
                + ", '" + photo.getPathFilter() + "' "
                + ", " + photo.getLatitude()
                + ", " + photo.getLongitude()
                + ", '" + photo.getComment() + "' "
                + ", " + photo.getStatus()
                + ", " + photo.getTime() + ")");
        db.close();
        return Constants.SUCCESS_SAVED;
    }
    @Override
    public Photo getPhoto(Context c, String path) {
        SQLiteDatabase db = getReadableDatabase();
         String[] CAMPOS = {"_id", "owner", "path", "pathFilter",
                 "latitude", "longitude", "comment", "status", "time"};
         Cursor cursor = db.query(mTableFullName, CAMPOS, "path='"+ path +"'", null,
                 null, null, null, null);

         Photo photoResult = new Photo();
         while (cursor.moveToNext()) {
             photoResult.setId(cursor.getInt(0));
             photoResult.setOwner(cursor.getString(1)); // owner
             photoResult.setPath(cursor.getString(2)); // path
             photoResult.setPathFilter(cursor.getString(3)); // pathFilter
             photoResult.setLatitude(cursor.getDouble(4)); // lat
             photoResult.setLongitude(cursor.getDouble(5)); // long
             photoResult.setComment(cursor.getString(6)); // comment
             photoResult.setStatus(cursor.getInt(7)); // status
             photoResult.setTime(cursor.getLong(8));// time
         }
         cursor.close();
         db.close();
         return photoResult;
    }
    @Override
    public List<Photo> getListPhotos(Context c) {
        SQLiteDatabase db = getReadableDatabase();
        String[] CAMPOS = {"_id",  "owner", "path", "pathFilter",
                "latitude", "longitude", "comment", "status", "time"};
        Cursor cursor=db.query(mTableFullName, CAMPOS, null, null,
                null, null, "time DESC", null);
        List<Photo> listPhotos = new ArrayList<>();
        while (cursor.moveToNext()){
            listPhotos.add(new Photo(
                    cursor.getInt(0),  //_id
                    cursor.getString(1), //owner
                    cursor.getString(2), // path
                    cursor.getString(3), // pathFilter
                    cursor.getDouble(4), // lat
                    cursor.getDouble(5), // long
                    cursor.getString(6), // comment
                    cursor.getInt(7), // status
                    cursor.getLong(8) // time
            ));
        }
        cursor.close();
        db.close();
        return listPhotos;
    }
    @Override
    public String deletePhotosByPath(String[] selectArg) {
        SQLiteDatabase db = getWritableDatabase();
        for(int i= 0; i < selectArg.length; i++) {
            if (selectArg[i] != null) {
                db.execSQL("DELETE FROM "+mTableFullName+" " +
                        "WHERE path='" + selectArg[i] + "'");
          }
        }
        db.close();
        return  Constants.SUCCESS_DELETED;
    }

    @Override
    public String deleteBinary(Context context, Uri uri)  {
        return UtilsFile.deleteBinaryPhotoFile(context, String.valueOf(uri));
    }

    @Override
    public String updatePhoto(Context mContext, Photo photo) {
        SQLiteDatabase db = getWritableDatabase();
        db.execSQL("UPDATE "+mTableFullName+" "
                        + "SET latitude='" + photo.getLatitude() + "', "
                        + "longitude='" + photo.getLongitude() + "', "
                        + "comment='" + photo.getComment() + "' "
                        + "WHERE path ='" + photo.getPath() + "'"
        );
        db.close();
        return Constants.SUCCESS_UPDATED;
    }
    @Override
    public String updatePathFilter(String[] selectsPaths) {
        SQLiteDatabase db = getWritableDatabase();
        db.execSQL("UPDATE "+mTableFullName+" "
                        + "SET pathFilter='" + selectsPaths[1] + "' "
                        + "WHERE path ='" + selectsPaths[0] + "'"
        );
        db.close();
        return Constants.SUCCESS_UPDATED;
    }

    @Override
    public String updateStatus(String[] args) {
        SQLiteDatabase db = getWritableDatabase();
        db.execSQL("UPDATE "+mTableFullName+" "
                        + "SET status='" + String.valueOf(args[1]) + "' "
                        + "WHERE path ='" + args[0] + "'"
        );
        db.close();
        return Constants.SUCCESS_UPDATED;
    }
    @Override
    public String uploadBinary(Context c, Uri uri) {
        return "Error SQLite in device, not upload";
    }
    public void showLogList(List<Photo> list){
        if(list.size() > 0) {
            for (int i = 0; i < list.size(); i++) {
                Utils.showLog(TAG, "LOG INFO list[" + i + "]: " + list.get(i).toString());
            }
        } else {
            Utils.showLog(TAG, "LOG INFO list.size<0");
        }
    }
}
