package com.jesusrojo.miphoto.model.almacen;

import android.content.Context;
import android.content.SharedPreferences;
import android.net.Uri;

import com.jesusrojo.miphoto.model.pojo.Photo;
import com.jesusrojo.miphoto.utils.Constants;
import com.jesusrojo.miphoto.utils.Utils;
import com.jesusrojo.miphoto.view.DatosConexionActivity;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.List;

public class PhotosWebService implements IAlmacen {

    private final String TAG = getClass().getSimpleName();
    private Context mContext;
    private SharedPreferences mSharedPref;
    private String hosting, register, login, upload, upload_binary,get_list,send_new_password;
    public final String CHARSET = "UTF-8";

    ////
    public PhotosWebService(Context mContext) {
        this.mContext = mContext;
        //datos de conexion desde shared preferences
        mSharedPref = mContext.getSharedPreferences(Constants.PREFERENCES_DEFAULT,
                Context.MODE_PRIVATE);
        hosting = mSharedPref.getString("hosting", "");
        register = mSharedPref.getString("register", "");
        login = mSharedPref.getString("login", "");
        upload = mSharedPref.getString("upload", "");
        upload_binary = mSharedPref.getString("upload_binary", "");
        get_list = mSharedPref.getString("get_list", "");
        send_new_password = mSharedPref.getString("send_new_password", "");
    }

    //MESSAGE DIGEST
    private String createMessageDigest( String password ){
        String resultPass = "";
        byte[] passB = password.getBytes();
        try {
            MessageDigest messageD = MessageDigest.getInstance("SHA-1");
            messageD.update(passB);
            byte[] bytes = messageD.digest();
            StringBuilder passS = new StringBuilder();
            for (byte b : bytes) {
                passS.append(String.format("%02x", b));
            }

            resultPass = passS.toString();
            return resultPass;

        } catch (NoSuchAlgorithmException e) {
            Utils.showLogError(TAG, "Error, e: " + e);
            return resultPass;
        }
    }

    //LINEA FROM HTML == RESPUESTA HTML
    private String getLineaHtml(HttpURLConnection conn)
            throws IOException {

        BufferedReader reader = new BufferedReader(
                new InputStreamReader(conn.getInputStream()));

        String linea = reader.readLine();
        Utils.showLog(TAG, "linea: " + linea);

        String serverRespuesta =linea.substring(0,1);
        Utils.showLog(TAG, "serverRespuesta: " + serverRespuesta);
        
        String miRespuesta = Constants.PHP_ERROR_CONEXION_ANOTHER;

        if (serverRespuesta.equals(Constants.PHP_SUCCESS)) {
            miRespuesta = Constants.SUCCESS;
        }
        if (serverRespuesta.equals(Constants.PHP_ERROR)) {
            miRespuesta =  Constants.ERROR;
        }
        if (serverRespuesta.equals(Constants.PHP_ERROR_CONEXION_DATABASE) ) {
            miRespuesta = Constants.PHP_ERROR_CONEXION_DATABASE;
        }
        if (serverRespuesta.equals(Constants.PHP_ERROR_FALSE)) {
            reader.close();
            miRespuesta = Constants.ERROR_FALSE;
        }
        if (serverRespuesta.equals(Constants.PHP_ERROR_ALREADY_USER)) {
            miRespuesta = Constants.ERROR_ALREADY_USER;
        }
        if (serverRespuesta.equals(Constants.PHP_ERROR_MORE_THAN_ONE)) {
            miRespuesta = Constants.ERROR_MORE_THAN_ONE;
        }
        if (serverRespuesta.equals(Constants.PHP_ERROR_EMPTY)) {
            miRespuesta = Constants.ERROR_EMPTY;
        }
        if (serverRespuesta.equals(Constants.PHP_ERROR_NO_USER)) {
            miRespuesta = Constants.ERROR_NO_USER;
        }
        if (serverRespuesta.equals(Constants.PHP_ERROR_FILE_SIZE)) {
            miRespuesta = Constants.ERROR_FILE_SIZE;
        }
        if (serverRespuesta.equals("")) {
            miRespuesta = Constants.ERROR_LECTURA_PRIMERA_LINEA_HTML_VACIA;
        }
        //close the reader
        reader.close();
        return miRespuesta;
    }
    //HTTP
    @Override
    public String register(String name, String password, String mail) {
        Utils.showLog(TAG, "register(");
        String respuestaPhp = Constants.PHP_ERROR_CONEXION_ANOTHER;

        password = createMessageDigest(password);

        if (!DatosConexionActivity.checkDatosConexion(mSharedPref)) {
            Utils.showLogError(TAG, "Error hosting");
            respuestaPhp = Constants.ERROR_NO_DATA_HOSTING;
            return respuestaPhp;

        } else {    //CONEXION
            URL url;
            HttpURLConnection conn;
            try {
                url = new URL(hosting + register);

                conn = (HttpURLConnection) url.openConnection();
                conn.setDoInput(true); // Allow Inputs
                conn.setDoOutput(true); // Allow Outputs
                conn.setUseCaches(false); // Don't use a Cached Copy
                conn.setRequestMethod("POST");
                conn.setConnectTimeout(60000);//tiempo conexion antes de cancelar
                conn.setReadTimeout(60000);// tiempo de lectura antes de canclear
                //DATA
                OutputStreamWriter sal = new OutputStreamWriter(
                        conn.getOutputStream());
                sal.write("name=");
                sal.write(URLEncoder.encode(name, CHARSET));
                sal.write("&password=");
                sal.write(URLEncoder.encode(password, CHARSET));
                sal.write("&mail=");
                sal.write(URLEncoder.encode(mail, CHARSET));
                sal.flush();
                ///
                if (conn.getResponseCode() == HttpURLConnection.HTTP_OK) {

                    respuestaPhp = getLineaHtml(conn);

                } else {
                    Utils.showLogError(TAG, "Error: " + conn.getResponseMessage());
                    respuestaPhp = Constants.ERROR_HTTP_NO_CONEXION;
                }
                conn.disconnect();

            } catch (Exception e) {
                Utils.showLogError(TAG, "Error, e : " + e);
                respuestaPhp = Constants.ERROR_EXCEPTION;
            }
        }
        return respuestaPhp;
    }
    @Override
    public String checkLogin(String name, String password) {
        Utils.showLog(TAG, "loginCloud(");

        String respuestaPhp = Constants.ERROR;
        password = createMessageDigest(password);

        if (!DatosConexionActivity.checkDatosConexion(mSharedPref)) {
            Utils.showLogError(TAG, "Error login");
            respuestaPhp =Constants.ERROR_NO_DATA_HOSTING;
            return respuestaPhp;

        } else {    //CONEXION
            Utils.showLog(TAG, "hosting + login " + hosting + login);
            URL url;
            HttpURLConnection conn;
            try {
                url = new URL( hosting + login );
                conn = (HttpURLConnection) url.openConnection();
                conn.setDoInput(true); // Allow Inputs
                conn.setDoOutput(true); // Allow Outputs
                conn.setUseCaches(false); // Don't use a Cached Copy
                conn.setRequestMethod("POST");
                conn.setConnectTimeout(60000);//tiempo conexion antes de cancelar
                conn.setReadTimeout(60000);// tiempo de lectura antes de canclear
                //DATA
                OutputStreamWriter sal = new OutputStreamWriter(
                        conn.getOutputStream());
                sal.write("name=");
                sal.write(URLEncoder.encode(name, CHARSET));
                sal.write("&password=");
                sal.write(URLEncoder.encode(password, CHARSET));
                sal.flush();
                ///
                if (conn.getResponseCode() == HttpURLConnection.HTTP_OK) {

                        respuestaPhp = getLineaHtml(conn);

                } else {
                    Utils.showLogError(TAG, "Error " + conn.getResponseMessage());
                    return Constants.ERROR_NOT_LOGIN;
                }
                conn.disconnect();

            } catch (Exception e) {
                Utils.showLogError(TAG, "Error e " + e);
                return Constants.ERROR_NOT_LOGIN;
            }
        }
        return respuestaPhp;
    }
    
    @Override
    public String[] sendNewPassword(String mail) {

        String[] args = new String[2];

        if (!DatosConexionActivity.checkDatosConexion(mSharedPref)) {
            Utils.showLogError(TAG, "Error hosting");
            return null;

        } else {    //CONEXION
            URL url;
            HttpURLConnection conn;
            try {
                url = new URL(hosting + send_new_password );

                conn = (HttpURLConnection) url.openConnection();
                conn.setDoInput(true); // Allow Inputs
                conn.setDoOutput(true); // Allow Outputs
                conn.setUseCaches(false); // Don't use a Cached Copy
                conn.setRequestMethod("POST");
                conn.setConnectTimeout(60000);//tiempo conexion antes de cancelar
                conn.setReadTimeout(60000);// tiempo de lectura antes de canclear
                //DATA
                OutputStreamWriter sal = new OutputStreamWriter(
                        conn.getOutputStream());
                sal.write("mail=");
                sal.write(URLEncoder.encode(mail, CHARSET));
                sal.flush();
                ///
                if (conn.getResponseCode() == HttpURLConnection.HTTP_OK) {
                    //respuestaPhp = getLineaHtml(conn);
                    Utils.showLog(TAG, "HTTP_OK");
                    BufferedReader reader = new BufferedReader(
                            new InputStreamReader(conn.getInputStream()));

                    String linea = reader.readLine();
                    while (linea != null && !linea.equals("")) {
                       // Utils.showLog(TAG, "linea: " + linea);

                        String[] tokens = linea.split(";");
                        args[0] =tokens[0];
                        args[1] =tokens[1];

                        linea = reader.readLine();
                    }
                    reader.close();
                    return args;
                } else {
                    Utils.showLogError(TAG, "Error: " + conn.getResponseMessage());
                }
                conn.disconnect();

            } catch (Exception e) {
                Utils.showLogError(TAG, "Error, e : " + e);
            }
        }
        return args;
    }

    @Override
    public String savePhoto(Context c, Photo photo) {
        String respuestaPhp = Constants.ERROR;

        if (!DatosConexionActivity.checkDatosConexion(mSharedPref)) {
            Utils.showLogError(TAG, "Error hosting");
            respuestaPhp = Constants.ERROR_NO_DATA_HOSTING;
            return respuestaPhp;
        } else {
            URL url;
            HttpURLConnection conn;
            try {

                url = new URL(hosting + upload);
                conn = (HttpURLConnection) url.openConnection();
                conn.setDoInput(true); // Allow Inputs
                conn.setDoOutput(true); // Allow Outputs
                conn.setUseCaches(false); // Don't use a Cached Copy
                conn.setRequestMethod("POST");
                conn.setConnectTimeout(60000);//tiempo conexion antes de cancelar
                conn.setReadTimeout(60000);// tiempo de lectura antes de canclear

                //send the DATA1 that fill the form
                OutputStreamWriter sal = new OutputStreamWriter(
                        conn.getOutputStream());
                sal.write("id=");
                sal.write( URLEncoder.encode( String.valueOf(photo.getId()) , CHARSET));
                sal.write("&owner=");
                sal.write(URLEncoder.encode(photo.getOwner(), CHARSET));
                sal.write("&path=");
                sal.write(URLEncoder.encode(photo.getPath(), CHARSET));
                sal.write("&pathFilter=");
                sal.write(URLEncoder.encode(photo.getPathFilter(), CHARSET));
                sal.write("&latitude=");
                sal.write(URLEncoder.encode(String.valueOf(photo.getLatitude()), CHARSET));
                sal.write("&longitude=");
                sal.write(URLEncoder.encode(String.valueOf(photo.getLongitude()), CHARSET));
                sal.write("&comment=");
                sal.write(URLEncoder.encode(photo.getComment(), CHARSET));
                sal.write("&status=");
                sal.write(URLEncoder.encode(String.valueOf(photo.getTime()), CHARSET));
                sal.write("&time=");
                sal.write(URLEncoder.encode(String.valueOf(photo.getTime()), CHARSET));
                sal.flush();

                if (conn.getResponseCode() == HttpURLConnection.HTTP_OK) {
                    Utils.showLog(TAG, "POJO okokokokokookok");
                    respuestaPhp = getLineaHtml( conn);

                } else {
                    Utils.showToast(mContext, "Error: " + conn.getResponseMessage());
                    respuestaPhp = Constants.ERROR_HTTP_NO_CONEXION;
                }
                conn.disconnect();

            } catch (Exception e) {
                Utils.showLogError(TAG, "Error save e : " + e);
                respuestaPhp = Constants.ERROR;
            }
            return respuestaPhp;
        }
    }
    @Override
    public String uploadBinary(Context c, Uri uri) {

        String respuestaPhp = Constants.ERROR;

        String fileName = String.valueOf(uri).substring(8);
        // UtilsFile.logBytes(new File(fileName));
        URL url;
        HttpURLConnection conn = null;
        DataOutputStream dos = null;
        String lineEnd = "\r\n";
        String twoHyphens = "--";
        String boundary = "*****";
        int bytesRead, bytesAvailable, bufferSize;
        byte[] buffer;
        int maxBufferSize = 1 * 1024 * 1024;
        File sourceFile = new File(fileName);
        if (!sourceFile.isFile()) {
            // dialog.dismiss();
            Utils.showLog(TAG, "Source File not exist ");
            return Constants.ERROR;
        }else {
            try {
                url = new URL(hosting + upload_binary);
                conn = (HttpURLConnection) url.openConnection();
                conn.setDoInput(true); // Allow Inputs
                conn.setDoOutput(true); // Allow Outputs
                conn.setUseCaches(false); // Don't use a Cached Copy
                conn.setRequestMethod("POST");
                conn.setConnectTimeout(5000);//tiempo conexion antes de cancelar
                conn.setReadTimeout(10000);// tiempo de lectura antes de canclear
                ///

                //////////////////////// IMAGE /////////////////////
                conn.setRequestProperty("Connection", "Keep-Alive");
                conn.setRequestProperty("ENCTYPE", "multipart/form-data");
                conn.setRequestProperty("Content-Type", "multipart/form-data;boundary=" + boundary);
                conn.setRequestProperty("archivo", fileName);

                dos = new DataOutputStream(conn.getOutputStream());
                dos.writeBytes(twoHyphens + boundary + lineEnd);
                dos.writeBytes("Content-Disposition: form-data; name=\"archivo\";filename=\""
                        + fileName + "\"" + lineEnd);
                dos.writeBytes(lineEnd);

                FileInputStream fileInputStream = new FileInputStream(sourceFile);
                bytesAvailable = fileInputStream.available();

                bufferSize = Math.min(bytesAvailable, maxBufferSize);
                buffer = new byte[bufferSize];
                // read file and write it into form...
                bytesRead = fileInputStream.read(buffer, 0, bufferSize);
                while (bytesRead > 0) {
                    dos.write(buffer, 0, bufferSize);
                    bytesAvailable = fileInputStream.available();
                    bufferSize = Math.min(bytesAvailable, maxBufferSize);
                    bytesRead = fileInputStream.read(buffer, 0, bufferSize);
                }
                // send multipart form data necesssary after file data...
                dos.writeBytes(lineEnd);
                dos.writeBytes(twoHyphens + boundary + twoHyphens + lineEnd);
                //end IMAGE

                // Responses from the server (code and message)
                int serverResponseCode = conn.getResponseCode();
                String serverResponseMessage = conn.getResponseMessage();

                Utils.showLog(TAG, "HTTP Response is : "
                        + serverResponseMessage + ": " + serverResponseCode);

                if(serverResponseCode == 200){
                    Utils.showLog(TAG, "BINARY okokookokokookokokoko");
                    respuestaPhp = getLineaHtml(conn);
                    ///
                } else {
                    Utils.showLogError(TAG, "Error fail, not 200");
                }
                fileInputStream.close();
                dos.flush();
                dos.close();
            } catch (MalformedURLException ex) {
                Utils.showLogError(TAG, "Error, Ex: " + ex);
            } catch (Exception e) {
                Utils.showLogError(TAG, "Error, e: " + e);
            }
            return respuestaPhp;
        } // End else block

    }
    @Override
    public List<Photo> getListPhotos(Context c) {

        List<Photo> listPhotos = new ArrayList<>();

        if (!DatosConexionActivity.checkDatosConexion(mSharedPref)) {
            Utils.showLogError(TAG, "Error login");
            return null;
        } else {    //CONEXION
            URL url;
            HttpURLConnection conn;
            try {

                url = new URL(hosting + get_list);
                conn = (HttpURLConnection) url.openConnection();
                conn.setDoInput(true); // Allow Inputs
                conn.setDoOutput(true); // Allow Outputs
                conn.setUseCaches(false); // Don't use a Cached Copy
                conn.setRequestMethod("POST");
                conn.setConnectTimeout(5000);//tiempo conexion antes de cancelar
                conn.setReadTimeout(10000);// tiempo de lectura antes de canclear

                if (conn.getResponseCode() == HttpURLConnection.HTTP_OK) {
                    Utils.showLog(TAG, "HTTP_OK");
                    BufferedReader reader = new BufferedReader(
                            new InputStreamReader(conn.getInputStream()));
                    String linea = reader.readLine();
                    while (linea != null && !linea.equals("")) {
                        Utils.showLog(TAG, "linea: " + linea);
                        ////////////////////////////////
                        //division y construccion de objeto
                        Photo photo = null;
                        String[] tokens = linea.split(";");
                        photo = new Photo(
                                Integer.valueOf(tokens[0]),
                                tokens[1],
                                tokens[2],
                                tokens[3],
                                Double.parseDouble(tokens[4]),
                                Double.parseDouble(tokens[5]),
                                tokens[6],
                                Integer.parseInt(tokens[7]),
                                Long.valueOf(tokens[8])
                        );
                        listPhotos.add(photo);
                        ////////////////////////////////
                        linea = reader.readLine();
                    }
                    reader.close();
                    return listPhotos;
                    //END HTTP OK///////////////////////////////////////
                } else {
                    Utils.showLogError(TAG, "Error conexion http: " + conn.getResponseMessage());
                }
                conn.disconnect();

            } catch (Exception e) {
                Utils.showLogError(TAG, "Error Exception e : " + e);
                return listPhotos;
            }
        }
        return listPhotos;
    }
    ///////////////////////////////////////////////null en servicio web
    @Override
    public Photo getPhoto(Context c, String path) {
        return null;
    }
    @Override
    public String updatePhoto(Context mContext, Photo photo) {return "Error WebService";}
    @Override
    public String deletePhotosByPath(String[] selectArg) {return  "Error WebService";}
    @Override
    public String deleteBinary(Context context,Uri selectPaths) { return "Error WebService";}
    @Override
    public String updatePathFilter(String[] pathSelects) {return "Error WebService";}
    @Override
    public String updateStatus(String[] pathSelects) {return "Error WebService";}
}
