package com.jesusrojo.miphoto.presenter;


import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnCancelListener;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.AsyncTask;
import android.widget.CheckBox;

import com.jesusrojo.miphoto.R;
import com.jesusrojo.miphoto.model.Mediator;
import com.jesusrojo.miphoto.utils.Constants;
import com.jesusrojo.miphoto.utils.Utils;
import com.jesusrojo.miphoto.view.LoginActivity;
import com.jesusrojo.miphoto.view.PhotosListActivity;

public class LoginTask extends AsyncTask<String, Void, String> {
	
	private final String TAG = getClass().getSimpleName();
    private Activity mActivity;
    private Context mContext;
    private SharedPreferences mSharedPref;
    private SharedPreferences.Editor mEditor;

    private ProgressDialog progressDialog;
    private Mediator mMediator;

	private String mName;
	private String mPassword;
    private String mFromWhere;

    public ILoginTaskCallback listener;
    public interface ILoginTaskCallback{
        void onFinishMyLoginTask(String result);
    }
    ////
	public LoginTask(Activity a, Context c, ProgressDialog p,
                     ILoginTaskCallback listener) {
		this.mActivity = a;
        this.mContext = c;
		this.progressDialog = p;
        this.listener = listener;

        mSharedPref = mActivity.getSharedPreferences(Constants.PREFERENCES_DEFAULT, Context.MODE_PRIVATE);
        mMediator = new Mediator(mContext);
    }

    public void empezarTask(String name, String password, String fromWhere){
        mName = name;
        mPassword =password;
        if(fromWhere.equals("device")){
            mFromWhere = "device";
        }
        if(fromWhere.equals("cloud")){
            mFromWhere = "cloud";
        }
        this.execute(name,password);
    }
    
	@Override
	protected void onPreExecute() {
		super.onPreExecute();
		try {
            progressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
            if(mFromWhere.equals("device")){
                progressDialog.setMessage(mActivity.getResources()
                        .getString(R.string.iniciando_sesion));

            }
            if(mFromWhere.equals("cloud")){
                progressDialog.setMessage(mActivity.getResources()
                        .getString(R.string.connecting_with_internet));
            }
            progressDialog.setMessage(mActivity.getResources()
                    .getString(R.string.connecting_with_internet));
            progressDialog.setCancelable(true);
			progressDialog.show();
			progressDialog.setOnCancelListener(new OnCancelListener() {
				@Override
				public void onCancel(DialogInterface dialog) {
					cancel(true);
				}
			});
		} catch (Exception e) {
			Utils.showLogError(TAG, "Error e " + e);
		}
    }

	@Override
	protected String doInBackground(String... arg) {

        mEditor =mSharedPref.edit();
        mEditor.putBoolean("login_status", false);
        mEditor.apply();

	    if (!isCancelled()) {
            return mMediator.checkLogin(mContext, arg[0], arg[1], mFromWhere);
		}
		return null;
	}
	@Override
	protected void onPostExecute(String result) {
		super.onPostExecute(result);
        if(progressDialog.isShowing()) {
            progressDialog.dismiss();
        }
        LoginActivity.isTaskWorking = false;
        listener.onFinishMyLoginTask(result);
    }
}
