package com.jesusrojo.miphoto.presenter.services;


import android.app.IntentService;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Build;
import android.support.v4.app.NotificationCompat.Builder;
import android.support.v4.content.LocalBroadcastManager;

import com.jesusrojo.miphoto.R;
import com.jesusrojo.miphoto.model.Mediator;
import com.jesusrojo.miphoto.utils.Constants;
import com.jesusrojo.miphoto.utils.Utils;
import com.jesusrojo.miphoto.utils.UtilsFile;

import java.io.File;


public class DeleteBinaryIntentService extends IntentService {

    private static final String TAG =  DeleteBinaryIntentService.class.getSimpleName();
    /**
     * Custom Action that will be used to send Broadcast to the ListActivity.
     */
    public static final String ACTION_DELETE_BYNARY_RESPONSE =
        "com.jesusrojo.miphoto.presenter.services.DeleteBinaryIntentService.RESPONSE";

    private static final int NOTIFICATION_ID = 10;
    // Create Mediator that will mediate the communication
    // between Server and Android Storage.
    private Mediator mMediator;
    // Manages the Notification displayed in System UI.
    private NotificationManager mNotifyManager;
    //Builder used to build the Notification.
    private Builder mBuilder;
    private Builder mBuilderCompat;
    //empty pending intent for gingerbread and bellow
    private PendingIntent pendingIntent;

    ////
    public DeleteBinaryIntentService(String name) {
        super("DeleteBinaryIntentService");
    }
    public DeleteBinaryIntentService() {
        super("DeleteBinaryIntentService");
    }

    public static Intent makeIntent(Context context, Uri uri, int index) {
        return new Intent(context,DeleteBinaryIntentService.class)
                .setData(uri)
                .putExtra("index", index);
    }

    @Override
    protected void onHandleIntent(Intent intent) {

        final Intent emptyIntent = new Intent();
        pendingIntent = PendingIntent.getActivity(this, 114,
                emptyIntent, PendingIntent.FLAG_UPDATE_CURRENT);


        Uri uri = intent.getData();
        String path = String.valueOf(uri);
        int index = intent.getIntExtra("index", 0);
        // Starts the Notification to show the progress of  upload.
        startNotification(index);

        mMediator = new Mediator(getApplicationContext());

         String respuesta = mMediator.deleteBinaryPhoto(
                 getApplicationContext(),uri);
        if(respuesta.equals(Constants.SUCCESS)){
            respuesta = Constants.SUCCESS_DELETED;
        }

        finishNotification(respuesta, index);
        // Send the Broadcast to ListActivity that the  Upload is completed.
        sendBroadcast(respuesta, path, index);
        //to see each notification minimum 2000
      //  Utils.sleepMiliSeconds(2000);
    }

    /**
     * Send the Broadcast to Activity that the Upload is completed.
     */
    private void sendBroadcast(String respuesta, String path, int index){
        // Use a LocalBroadcastManager to restrict the scope of this
        // Intent to the  application.
        LocalBroadcastManager.getInstance(this)
             .sendBroadcast(new Intent(ACTION_DELETE_BYNARY_RESPONSE)
                     .putExtra("respuesta", respuesta)
                     .putExtra("path", path)
                     .putExtra("index", index)
                     .addCategory(Intent.CATEGORY_DEFAULT));
    }
    private void finishNotification(String status, int index) {
        String myTicker = status + " " + index;
        if (Build.VERSION.SDK_INT < 11 ) {
            //NotificationCompact.Builder pertenece a la lib compatibilidadd v4
            mBuilderCompat = new Builder(this)
                    .setContentTitle(getResources().getString(R.string.app_name))
                    .setSmallIcon(android.R.drawable.stat_sys_upload_done)
                    .setContentText(status)
                    .setWhen(System.currentTimeMillis() + 1000 * 60 * 60)
                    .setTicker(myTicker)
                    .setProgress(0, 0, false);// Removes progress bar
            mBuilderCompat.setContentIntent(pendingIntent);
            // end mBuilderCompat
            mNotifyManager.notify(NOTIFICATION_ID,mBuilderCompat.build());
        } else {    //API + 11
            // When the loop is finished, updates the notification.
            mBuilder.setContentTitle(getResources().getString(R.string.app_name))
                    .setSmallIcon(android.R.drawable.stat_sys_upload_done)
                    .setContentText(status)
                    .setTicker(myTicker)
                    .setWhen(System.currentTimeMillis() + 1000 * 60 * 60)
                    .setProgress(0, 0, false);// Removes progress bar
            // Build the Notification with the given Notification Id.
            mNotifyManager.notify(NOTIFICATION_ID, mBuilder.build());
    }
}
    private void startNotification(int count) {
        mNotifyManager = (NotificationManager)
            getSystemService(Context.NOTIFICATION_SERVICE);

        String myTicker = getResources().getString(R.string.deleting) + "  " + count;

        if (Build.VERSION.SDK_INT < 11 ) {
            //START NotificationCompact.Builder pertenece a la lib compatibilidadd v4
           mBuilderCompat = new Builder(this)
                       .setContentTitle(getResources().getString(R.string.app_name))
                   .setContentText(getResources().getString(R.string.delete_in_progress))
                   .setSmallIcon(android.R.drawable.stat_sys_upload)
                   .setTicker(myTicker)
                   .setWhen(System.currentTimeMillis() + 1000 * 60 * 60)
                           //.setSound(Uri.parse("android.resource://"+getPackageName()+R.raw.explosion))
                   .setProgress(0, 0, true)
                   .setContentIntent(pendingIntent);
            ;//end mBuilderCompat
            mNotifyManager.notify(NOTIFICATION_ID,mBuilderCompat.build());
        } else {    //API + 11
            // Create the Notification and set a progress indicator for an
            // operation of indeterminate length.
            mBuilder = new Builder(this)
                    .setContentTitle(getResources().getString(R.string.app_name))
                    .setContentText(getResources().getString(R.string.delete_in_progress))
                    .setSmallIcon(android.R.drawable.stat_sys_upload)
                    .setTicker(myTicker)
                    .setWhen(System.currentTimeMillis() + 1000 * 60 * 60)
                    .setProgress(0, 0, true);
            // Build and issue the notification.
            mNotifyManager.notify(NOTIFICATION_ID, mBuilder.build());
        }
    }
    @Override
    public void onDestroy() {
        Utils.showLog(TAG, "Servicio SEEK onDestroy()");
        //si queremos eliminar la notificacion al finalizar el servicio
       // NotificationManager notificationManager = (NotificationManager)getSystemService(Context.NOTIFICATION_SERVICE);
        mNotifyManager.cancel(NOTIFICATION_ID);
        super.onDestroy();
    }
}
