package com.jesusrojo.miphoto.presenter.services;


import android.app.IntentService;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.net.Uri;
import android.support.v4.app.NotificationCompat.Builder;
import android.support.v4.content.LocalBroadcastManager;

import com.jesusrojo.miphoto.R;
import com.jesusrojo.miphoto.model.Mediator;
import com.jesusrojo.miphoto.utils.Constants;
import com.jesusrojo.miphoto.utils.Utils;
import com.jesusrojo.miphoto.utils.UtilsFile;
import com.jesusrojo.miphoto.utils.UtilsFilter;
import com.jesusrojo.miphoto.view.MainActivity;
import com.jesusrojo.miphoto.view.PhotosListActivity;

import java.io.File;


public class FilterIntentService extends IntentService {

    private static final String TAG =  FilterIntentService.class.getSimpleName();
    private Context mContext;
    public static final String ACTION_FILTER_RESPONSE =
        "com.jesusrojo.miphoto.presenter.services.FilterIntentService.RESPONSE";

    private static final int NOTIFICATION_ID_FILTER = 141;
    private Mediator mMediator;
    private NotificationManager mNotifyManager;
    private Builder mBuilderCompat;
    private static File mFile;
    //empty pending intent for gingerbread and bellow
    private PendingIntent pendingIntent;

    ////
    public FilterIntentService(String name) {
        super("FilterIntentService");
    }
    public FilterIntentService() {
        super("FilterIntentService");
    }

    public static Intent makeIntent(Context context, Uri uri) {
        return new Intent(context,FilterIntentService.class).setData(uri);
    }

    @Override
    protected void onHandleIntent(Intent intent) {
        final Intent emptyIntent = new Intent();
        pendingIntent = PendingIntent.getActivity(this, 114,
                emptyIntent, PendingIntent.FLAG_UPDATE_CURRENT);
        // Starts the Notification to show the progress of  upload.
        startNotification();

        //creamos el archivo donde vamos a guardar la binary data
        String fileName = "filter_" + Utils.getDiaYHora() + ".jpg";
        File file = UtilsFile.createFileInMyFolder(getApplicationContext(),
                PhotosListActivity.folder_filter, fileName);
        String pathFiltered = "file://"+file.getPath();

        //path recibida
        Uri uri =intent.getData();
        String path = String.valueOf(uri);

        String respuesta = Constants.ERROR_NOT_FILTERED;
        /*1*/
        Bitmap bitmap =  new Mediator().getFilteredBitmap(getApplicationContext(),
                uri);

        if(bitmap != null){
            respuesta = Constants.SUCCESS_FILTER;
            UtilsFile.saveBitmapInFile(file,bitmap);
        }
        /*
        Uri uriFiltered = UtilsFilter.grayScaleFilter(getApplicationContext(),uri);
        pathFiltered = String.valueOf(uriFiltered);
        if(uriFiltered != null){
            respuesta = Constants.SUCCESS_FILTER;
        }
        */


        finishNotification(respuesta);
        // Send the Broadcast to ListActivity that the  Upload is completed.
        sendBroadcast(respuesta, path,pathFiltered);
        //to see each notification minimum 2000
        Utils.sleepMiliSeconds(2000);
    }

    /**
     * Send the Broadcast to Activity that the Upload is completed.
     */
    private void sendBroadcast(String respuesta, String path, String pathFiltered){
        // Use a LocalBroadcastManager to restrict the scope of this
        // Intent to the  application.
        LocalBroadcastManager.getInstance(this)
             .sendBroadcast(new Intent(ACTION_FILTER_RESPONSE)
                     .putExtra("respuesta", respuesta)
                     .putExtra("path", path)
                     .putExtra("pathFiltered", pathFiltered)
                     .addCategory(Intent.CATEGORY_DEFAULT));
    }
    private void finishNotification(String status) {
            //NotificationCompact.Builder pertenece a la lib compatibilidadd v4
            mBuilderCompat = new Builder(this)
                    .setContentTitle(getResources().getString(R.string.app_name))
                    .setSmallIcon(android.R.drawable.stat_sys_upload_done)
                    .setContentText(status)
                    .setWhen(System.currentTimeMillis() + 1000 * 60 * 60)
                    .setTicker(status)
                    .setProgress(0, 0, false);// Removes progress bar
            mBuilderCompat.setContentIntent(pendingIntent);
            mNotifyManager.notify(NOTIFICATION_ID_FILTER,mBuilderCompat.build());
    }
    private void startNotification() {
        Utils.showLog(TAG, "startNotification()");
        mNotifyManager = (NotificationManager)
            getSystemService(Context.NOTIFICATION_SERVICE);
        //START NotificationCompact.Builder pertenece a la lib compatibilidadd v4
        mBuilderCompat = new Builder(this)
                   .setContentTitle(getResources().getString(R.string.app_name))
               .setContentText(getResources().getString(R.string.filter_in_progress))
               .setSmallIcon(android.R.drawable.stat_sys_upload)
               .setTicker(getResources().getString(R.string.filtering))
               .setWhen(System.currentTimeMillis() + 1000 * 60 * 60)
                       //.setSound(Uri.parse("android.resource://"+getPackageName()+R.raw.explosion))
               .setProgress(0, 0, true)
               .setContentIntent(pendingIntent);
        mNotifyManager.notify(NOTIFICATION_ID_FILTER,mBuilderCompat.build());
    }
    @Override
    public void onDestroy() {
        Utils.showLog(TAG, "Servicio SEEK onDestroy()");
        //si queremos eliminar la notificacion al finalizar el servicio
       // NotificationManager notificationManager = (NotificationManager)getSystemService(Context.NOTIFICATION_SERVICE);
        mNotifyManager.cancel(NOTIFICATION_ID_FILTER);
        super.onDestroy();
    }
}
