package com.jesusrojo.miphoto.presenter.services;


import android.app.IntentService;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.support.v4.app.NotificationCompat;
import android.support.v4.app.NotificationCompat.Builder;
import android.support.v4.content.LocalBroadcastManager;

import com.jesusrojo.miphoto.R;
import com.jesusrojo.miphoto.model.Mediator;
import com.jesusrojo.miphoto.utils.Constants;
import com.jesusrojo.miphoto.utils.Utils;


public class UploadPhotoIntentService extends IntentService {

    private static final String TAG =  UploadPhotoIntentService.class.getSimpleName();
    /**
     * Custom Action that will be used to send Broadcast to the ListActivity.
     */
    public static final String ACTION_UPLOAD_RESPONSE =
        "com.jesusrojo.miphoto.presenter.services.UploadPhotoIntentService.RESPONSE";

    private static final int NOTIFICATION_ID_UPLOAD = 1;
    // Create Mediator that will mediate the communication
    // between Server and Android Storage.
    private Mediator mMediator;
    // Manages the Notification displayed in System UI.
    private NotificationManager mNotifyManager;
    //Builder used to build the Notification.
    private Builder mBuilder;

    private NotificationCompat.Builder mBuilderCompat;
    //empty pending intent
    private PendingIntent mPendingIntent;

    ////
    public UploadPhotoIntentService(String name) {
        super("UploadPhotoIntentService");
    }
    public UploadPhotoIntentService() {
        super("UploadPhotoIntentService");
    }

    public static Intent makeIntent(Context context, Uri uri) {
        return new Intent(context,UploadPhotoIntentService.class).setData(uri);
    }

    @Override
    protected void onHandleIntent(Intent intent) {
        final Intent emptyIntent = new Intent();
        mPendingIntent = PendingIntent.getActivity(this, 14,
                emptyIntent, PendingIntent.FLAG_UPDATE_CURRENT);

        // Starts the Notification to show the progress of  upload.
        startNotification();

        mMediator = new Mediator(getApplicationContext());

        Uri uri = intent.getData();
        String path= String.valueOf(uri);
        String respuesta =
                mMediator.uploadPhoto(getApplicationContext(), uri);
        //to see each notification minimum 2000
        Utils.sleepMiliSeconds(2000);
        if(respuesta.equals(Constants.SUCCESS)){
            respuesta = Constants.SUCCESS_UPLOADED;
        }
        finishNotification(respuesta);
        // Send the Broadcast to ListActivity that the  Upload is completed.
        sendBroadcast(respuesta, path);
        //to see each notification minimum 2000
        Utils.sleepMiliSeconds(2000);
    }
      /**
     * Send the Broadcast to Activity that the Upload is completed.
     */
    private void sendBroadcast(String respuesta, String path){
        // Use a LocalBroadcastManager to restrict the scope of this
        // Intent to the  application.
        LocalBroadcastManager.getInstance(this)
             .sendBroadcast(new Intent(ACTION_UPLOAD_RESPONSE)
                     .putExtra("respuesta", respuesta)
                     .putExtra("path", path)
                     .addCategory(Intent.CATEGORY_DEFAULT));
    }
    private void finishNotification(String status) {
        // When the loop is finished, updates the notification.
        mBuilderCompat.setContentTitle(getResources().getString(R.string.app_name))
                .setProgress(0, 0, false)
                .setSmallIcon(android.R.drawable.stat_sys_upload_done)
                .setContentText(status)
                .setTicker(status)
                .setWhen(System.currentTimeMillis() + 1000 * 60 * 60);
        // Build the Notification with the given Notification Id.
        mNotifyManager.notify(NOTIFICATION_ID_UPLOAD, mBuilderCompat.build());
    }
    private void startNotification() {
        mNotifyManager = (NotificationManager)
            getSystemService(Context.NOTIFICATION_SERVICE);
        //START NotificationCompact.Builder pertenece a la lib compatibilidadd v4
        mBuilderCompat = new NotificationCompat.Builder(this)
                .setContentTitle(getResources().getString(R.string.app_name))
                .setContentText(getResources().getString(R.string.upload_in_progress))
                .setSmallIcon(android.R.drawable.stat_sys_upload)
                .setTicker(getResources().getString(R.string.uploading))
                .setProgress(0, 0, true)
                .setContentIntent(mPendingIntent);
        mNotifyManager.notify(NOTIFICATION_ID_UPLOAD,mBuilderCompat.build());
    }
    @Override
    public void onDestroy() {
        Utils.showLog(TAG, "Servicio SEEK onDestroy()");
        //si queremos eliminar la notificacion al finalizar el servicio
       // NotificationManager notificationManager = (NotificationManager)getSystemService(Context.NOTIFICATION_SERVICE);
        mNotifyManager.cancel(NOTIFICATION_ID_UPLOAD);
        super.onDestroy();
    }
}
