package com.jesusrojo.miphoto.utils;

import android.app.Activity;
import android.content.ContentResolver;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.Build;
import android.os.Environment;
import android.os.StatFs;
import android.provider.MediaStore;
import android.support.annotation.NonNull;
import android.util.Base64;
import android.widget.ImageView;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

public class UtilsFile {

    private static final String TAG = UtilsFile.class.getSimpleName();

    public static boolean comprobarSiHaySDCard(Context context) {
        String state = Environment.getExternalStorageState();

        if (Environment.MEDIA_MOUNTED.equals(state)) {
            if (comprobarSiHayEspacioEnSDCard(context)) {
                return true;
            } else {
                return false;
            }
        } else {
            Utils.showToast(context, "Error, SDCard is not mounted");
            Utils.showLogError(TAG, "Error SDCard is not mounted");
            return false;
        }
    }
    public static boolean comprobarSiHayEspacioEnSDCard(Context context) {
        // do this only *after* you have checked external storage state:
        File extdir = Environment.getExternalStorageDirectory();
        StatFs stats = new StatFs(extdir.getAbsolutePath());

        if (Build.VERSION.SDK_INT >= 18) {

            long lBytesAvailable = (long) stats.getAvailableBlocksLong() * (long) stats.getBlockSizeLong();
            float fBytesAvailable = lBytesAvailable / (1024.f * 1024.f);
           // Utils.showLog(TAG, "API>18  bytesAvailable: (long)" + lBytesAvailable);
          //  Utils.showLog(TAG,  "API>18 fBytesAvailable: (float) " + fBytesAvailable);

            if (fBytesAvailable > 0) {
              //  Utils.showLog(TAG, "OK SDCard size: (float) " + fBytesAvailable);
                return true;
            } else {
              //  Utils.showLog(TAG,"NO SDCard size:  (float) " + fBytesAvailable);
                return false;
            }
        } else {
            //  @SuppressWarnings("deprecation")
            double dAvailableBytes =
                    (double) stats.getAvailableBlocks() * (double) stats.getBlockSize();
          //  Utils.showLog(TAG,"api<18 dAvailableBytes: (double) " + dAvailableBytes);

            if (dAvailableBytes > 0) {
              //  Utils.showLog(TAG,"OK SDCard size: (double) " + dAvailableBytes);
                return true;
            } else {
             //   Utils.showLog(TAG, "NO SDCard size:  (double) " + dAvailableBytes);
                return false;
            }
        }
    }
    public static File createFileInMyFolder(Context context,
                                            String folderName,
                                            String fileName) {
        File file = null;
        if( UtilsFile.comprobarSiHaySDCard(context)
                && UtilsFile.comprobarSiHayEspacioEnSDCard(context) ){

            File STORAGE_DIR =
                    Environment.getExternalStoragePublicDirectory(
                            Environment.DIRECTORY_PICTURES
                                    + File.separator
                                    + folderName)                            ;

            if (fileName == null) {
                 fileName = "img_" + Utils.getDiaYHora() + ".jpg";
            }
            file = new File(STORAGE_DIR, fileName);
            if(!file.exists()) {
                file.getParentFile().mkdir();
                Utils.showLog(TAG, "created file: " + file.getAbsolutePath());
                Utils.showToast(context, file.getAbsolutePath());
            }
        }
        return file;
    }

    //DELETE THE REAL FILE === BINARY DATA ===
    public static String deleteBinaryPhotoFile(Context context, String path) {

        if(path != null) {
            File fileToDelete = new File(path.substring(8));
            File file = new File(fileToDelete.getAbsolutePath());
            if(file.exists()) {
                if (file.delete()) {
                    Utils.showLog(TAG, "Ok File deleted: " + path);
                    return Constants.SUCCESS;
                } else {
                    Utils.showLogError(TAG, "Error, file  not deleted: " + path);
                    return Constants.ERROR;
                }
            } else {
                Utils.showLogError(TAG, "Error file not exits: " + path);
                return Constants.ERROR;
            }
        } else {
            Utils.showLogError(TAG, "Error path null: " + path);
            return Constants.ERROR;
        }
    }
    @NonNull
    public static String copyInputStreamToMyFile(Context context, Uri uri, File file) {
        InputStream is = null;
        try {
            is = context.getContentResolver().openInputStream(uri);

            if (is != null ) {
                try {
                    OutputStream out = new FileOutputStream(file);
                    byte[] buf = new byte[1024];
                    int len;
                    while((len=is.read(buf))>0){
                        out.write(buf,0,len);
                    }
                    out.close();
                    is.close();
                    return Constants.SUCCESS_COPIED;
                } catch (Exception e) {
                    Utils.showLogError(TAG, "Error e: " + e);
                    return Constants.ERROR;
                }
            } else {
                return Constants.ERROR_NOT_COPIED;
            }
        } catch (FileNotFoundException e) {
            Utils.showLog(TAG, "Error1, e: " + e);
            return Constants.SUCCESS_COPIED;
        }
    }
    public static Uri getUriFromBitmap(Context context,Bitmap bitmap) {

        ContentValues values = new ContentValues();
        values.put(MediaStore.Images.Media.TITLE, "title");
        values.put(MediaStore.Images.Media.MIME_TYPE, "image/*");//jpeg
        Uri uri = context.getContentResolver().insert(
                MediaStore.Images.Media.EXTERNAL_CONTENT_URI,
                values);

        if(uri != null) {
            OutputStream outstream;
            try {
                outstream = context.getContentResolver().openOutputStream(uri);
                bitmap.compress(Bitmap.CompressFormat.JPEG, 100, outstream);
                if (outstream != null) {
                    outstream.close();
                }
            } catch (Exception e) {
                System.err.println(e.toString());
            }
            Utils.showLog(TAG, "getUriFromBitmap return uri " + uri);
            return uri;
        } else {
            return null;
        }
    }

    public static String getPath(Activity activity, Uri uri) {
        if( uri == null ) {
            Utils.showToast(activity, "Error, there is not file.");
            return null;
        }
        // try to retrieve the image from the media store first
        // this will only work for images selected from gallery
        String[] projection = { MediaStore.Images.Media.DATA };
        Cursor cursor = activity.managedQuery(uri, projection, null, null, null);
        if( cursor != null ){
            int column_index = cursor
                    .getColumnIndexOrThrow(MediaStore.Images.Media.DATA);
            cursor.moveToFirst();
            return cursor.getString(column_index);
        }
        // this is our fallback here
        return uri.getPath();
    }
    public static Bitmap getBitmapFromPath(String path) {
        Bitmap bitmap = null;
        if (path != null) {
            try {
                InputStream inputStream =
                        new FileInputStream(path.substring(8));
                //OUT OF MEMORY
                bitmap = BitmapFactory.decodeStream(inputStream);

            } catch (FileNotFoundException e) {
                Utils.showLogError(TAG, "Error e " + e);
                return bitmap;
            }
        }
        return bitmap;
    }
    public static void saveBitmapInFile(File file, Bitmap filteredBitmap) {
        FileOutputStream out = null;
        try {
            out = new FileOutputStream(file.getPath());
            filteredBitmap.compress(Bitmap.CompressFormat.PNG, 100, out); // bmp is your Bitmap instance
            // PNG is a lossless format, the compression factor (100) is ignored
        } catch (Exception e) {
            Utils.showLogError(TAG, "Error e " + e);
        } finally {
            try {
                if (out != null) {
                    out.close();
                }
            } catch (IOException e) {
                Utils.showLogError(TAG, "Error e " + e);
            }
        }
    }
    /**
     * Create a temporary filename to store the result of a download.
     * @param url Name of the URL.
     * @return String containing the temporary filename.
     */
    static private String getTemporaryFilename(final String url) {
        // This is what you'd normally call to get a unique temporary
        // filename, but for testing purposes we always name the file
        // the same to avoid filling up student phones with numerous
        // files!
        //
        //REAL
        return Base64.encodeToString(url.getBytes(),
                Base64.NO_WRAP)
                + System.currentTimeMillis()+ ".jpg";
        //ESCRIBE ENCIMA
        // return Base64.encodeToString(url.getBytes(),Base64.NO_WRAP) + ".jpg";
    }
    public static Bitmap decodeSampledBitmapFromFile(Context context, String path,
                                                     int reqWidth, int reqHeight){
        if( UtilsFile.comprobarSiHaySDCard(context)
                && UtilsFile.comprobarSiHayEspacioEnSDCard(context) ) {

            //First decode with inJustDecodeBounds=true to check dimensions
            final BitmapFactory.Options options = new BitmapFactory.Options();
            options.inJustDecodeBounds = true;
            BitmapFactory.decodeFile(path, options);

            // Calculate inSampleSize, Raw height and width of image
            final int height = options.outHeight;
            final int width = options.outWidth;
            options.inPreferredConfig = Bitmap.Config.RGB_565;
            int inSampleSize = 2;

            if (height > reqHeight) {
                inSampleSize = Math.round((float) height / (float) reqHeight);
            }
            int expectedWidth = width / inSampleSize;

            if (expectedWidth > reqWidth) {
                //if(Math.round((float)width / (float)reqWidth) > inSampleSize) // If bigger SampSize..
                inSampleSize = Math.round((float) width / (float) reqWidth);
            }

            options.inSampleSize = inSampleSize;

            // Decode bitmap with inSampleSize set
            options.inJustDecodeBounds = false;
            return BitmapFactory.decodeFile(path, options);
        } else {
            return  null;
        }
    }
    public static Bitmap reduceBitmap(Context contexto, String path,
                                      int maxAncho, int maxAlto) {
        try {
            final BitmapFactory.Options options = new BitmapFactory.Options();
            options.inJustDecodeBounds = true;

            BitmapFactory.decodeStream(contexto.getContentResolver()
                    .openInputStream(Uri.parse(path)), null, options);

            options.inSampleSize = (int) Math.max(
                    Math.ceil(options.outWidth / maxAncho),
                    Math.ceil(options.outHeight / maxAlto));
            options.inJustDecodeBounds = false;
            return BitmapFactory.decodeStream(contexto.getContentResolver()
                   .openInputStream(Uri.parse(path)), null, options);

        } catch (FileNotFoundException e) {
            // Utils.showToast(contexto,"Fichero/recurso no encontrado");
            Utils.showLogError(TAG, "Fichero/recurso no encontrado");
            e.printStackTrace();
            return null;
        }
    }
    public static Bitmap getThumbnail(String photoPath) {
        int THUMB_DIM = 50;

        // Get the dimensions of the View
        int targetW = THUMB_DIM;
        int targetH = THUMB_DIM;

        // Get the dimensions of the bitmap
        BitmapFactory.Options bmOptions = new BitmapFactory.Options();
        bmOptions.inJustDecodeBounds = true;
        BitmapFactory.decodeFile(photoPath, bmOptions);
        int photoW = bmOptions.outWidth;
        int photoH = bmOptions.outHeight;

        // Determine how much to scale down the image
        int scaleFactor = Math.min(photoW / targetW, photoH / targetH);

        // Decode the image file into a Bitmap sized to fill the View
        bmOptions.inJustDecodeBounds = false;
        bmOptions.inSampleSize = scaleFactor;
        bmOptions.inPurgeable = true;
        Bitmap bitmap = BitmapFactory.decodeFile(photoPath, bmOptions);
        return bitmap;
    }

    public static Bitmap grabImage(Context context, Uri uri, ImageView imageView) {
        context.getContentResolver().notifyChange(uri, null);
        ContentResolver cr = context.getContentResolver();
        Bitmap bitmap= null;
        try{
            bitmap = MediaStore.Images.Media.getBitmap(cr, uri);
            //imageView.setImageBitmap(bitmap);
        } catch (Exception e){
            Utils.showLogError(TAG, "Error e: " + e);
        }
        return bitmap;
    }

    public static  void galleryAddPic(Activity activity,String filePath) {
        Intent mediaScanIntent = new Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE);
        File f = new File(filePath);
        Uri contentUri = Uri.fromFile(f);
        mediaScanIntent.setData(contentUri);
        activity.sendBroadcast(mediaScanIntent);
    }

    public static Uri getImageUri(Context context, Bitmap inImage) {
        ByteArrayOutputStream bytes = new ByteArrayOutputStream();
        inImage.compress(Bitmap.CompressFormat.JPEG, 100, bytes);
        String path = MediaStore.Images.Media.insertImage(context.getContentResolver(), inImage, "Title", null);
        return Uri.parse(path);
    }
    public static void logBytes(File file){
        double bytes = file.length();
        double kilobytes = (bytes / 1024);
        double megabytes = (kilobytes / 1024);
        double gigabytes = (megabytes / 1024);
        double terabytes = (gigabytes / 1024);
        double petabytes = (terabytes / 1024);
        double exabytes = (petabytes / 1024);
        double zettabytes = (exabytes / 1024);
        double yottabytes = (zettabytes / 1024);
        Utils.showLog(TAG,"BYTES");
        Utils.showLog(TAG,"bytes : " + bytes);
        Utils.showLog(TAG,"kilobytes : " + kilobytes);
        Utils.showLog(TAG,"megabytes : " + megabytes);
        Utils.showLog(TAG,"gigabytes : " + gigabytes);
        Utils.showLog(TAG,"terabytes : " + terabytes);
        Utils.showLog(TAG,"petabytes : " + petabytes);
        Utils.showLog(TAG,"exabytes : " + exabytes);
        Utils.showLog(TAG,"zettabytes : " + zettabytes);
        Utils.showLog(TAG,"yottabytes : " + yottabytes);
    }

}
