package com.jesusrojo.miphoto.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.net.URL;
import java.util.Locale;

import android.app.Activity;
import android.content.ContentResolver;
import android.content.ContentValues;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.net.Uri;
import android.os.Environment;
import android.os.IBinder;
import android.provider.MediaStore.Images;
import android.util.Base64;
import android.util.Log;
import android.view.inputmethod.InputMethodManager;
import android.widget.Toast;

import com.jesusrojo.miphoto.R;

/**
 * This helper class encapsulates several static methods that are used
 * to download image files.
 */
public class UtilsFilter {
    /**
     * Used for debugging.
     */
    private final static String TAG = "UtilsFilter";

    public static Bitmap doGreyFilter(Bitmap src, Uri uri) {
        // constant factors
        final double GS_RED = 0.299;
        final double GS_GREEN = 0.587;
        final double GS_BLUE = 0.114;
        // create output bitmap
        // get image size
        int width = src.getWidth();
        int height = src.getHeight();
        ///////////////////
        // OutOfMemoryError  a veces
       // 1 ******************
       Bitmap bmOut = Bitmap.createBitmap(width, height
                                          //,src.getConfig()
                                           ,Bitmap.Config.RGB_565
                                           );
         /////////////////////
        // pixel information
        int A, R, G, B;
        int pixel;
        // scan through every single pixel
        for (int x = 0; x < width; ++x) {
            for (int y = 0; y < height; ++y) {
                // get one pixel color
                pixel = src.getPixel(x, y);
                // retrieve color of all channels
                A = Color.alpha(pixel);
                R = Color.red(pixel);
                G = Color.green(pixel);
                B = Color.blue(pixel);
                // take conversion up to one single value
                R = G = B = (int) (GS_RED * R + GS_GREEN * G + GS_BLUE * B);
                // set new pixel color to output bitmap
                bmOut.setPixel(x, y, Color.argb(A, R, G, B));
            }
        }
        return bmOut;
    }

    public  Bitmap doColorFilter(Bitmap src,
                                 double red, double green, double blue) {

        // image size
        int width = src.getWidth();
        int height = src.getHeight();
        // create output bitmap
        Bitmap bmOut = Bitmap.createBitmap(width, height, src.getConfig());
        // color information
        int A, R, G, B;
        int pixel;

        // scan through all pixels
        for(int x = 0; x < width; ++x) {
            for(int y = 0; y < height; ++y) {
                // get pixel color
                pixel = src.getPixel(x, y);
                // apply filtering on each channel R, G, B
                A = Color.alpha(pixel);
                R = (int)(Color.red(pixel) * red);
                G = (int)(Color.green(pixel) * green);
                B = (int)(Color.blue(pixel) * blue);
                // set new color pixel to output bitmap
                bmOut.setPixel(x, y, Color.argb(A, R, G, B));
            }
        }
        return bmOut;
    }

    public static Uri grayScaleFilter(Context context,Uri pathToImageFile) {
        Utils.showLog(TAG, "grayScaleFilter(" +pathToImageFile.toString().substring(8));
        Bitmap grayScaleImage = null;

        InputStream inputStream = null;
        try {
            inputStream = new FileInputStream(pathToImageFile.toString().substring(8));
        } catch (FileNotFoundException e) {
            Utils.showLogError(TAG, "Error e " + e);
        }
        //added
       // BitmapFactory.Options options = new BitmapFactory.Options();
       // options.inJustDecodeBounds = true;
      //  options.inPreferredConfig = Bitmap.Config.RGB_565;
       // Bitmap originalImage = BitmapFactory.decodeStream(inputStream,null, options);
        //end added

        //original
         Bitmap originalImage = BitmapFactory.decodeStream(inputStream);

            // Bail out of we get an invalid bitmap.
            if (originalImage == null){
                return null;
            }

        grayScaleImage = originalImage.copy(originalImage.getConfig(),true);
        boolean hasTransparent = grayScaleImage.hasAlpha();
        int width = grayScaleImage.getWidth();
        int height = grayScaleImage.getHeight();
        // A common pixel-by-pixel grayscale conversion algorithm
        // using values obtained from en.wikipedia.org/wiki/Grayscale.
        for (int i = 0; i < height; ++i) {
            for (int j = 0; j < width; ++j) {

            	// Check if the pixel is transparent in the original
            	// by checking if the alpha is 0
                if (hasTransparent 
                    && ((grayScaleImage.getPixel(j, i) & 0xff000000) >> 24) == 0) {
                    continue;
                }
                // Convert the pixel to grayscale.
                int pixel = grayScaleImage.getPixel(j, i);
                int grayScale = 
                    (int) (Color.red(pixel) * .299 
                           + Color.green(pixel) * .587
                           + Color.blue(pixel) * .114);
                grayScaleImage.setPixel(j, i, 
                                     Color.rgb(grayScale, grayScale, grayScale)
                                     );
            }
        }
        return UtilsFilter.createDirectoryAndSaveFile
                (context,
                        grayScaleImage,
                        // Name of the image file that we're filtering.
                        pathToImageFile.toString());
    }

    /**
     * Decode an InputStream into a Bitmap and store it in a file on
     * the device.
     *
     * @param context	   the context in which to write the file.
     * @param image        the image to save
     * @param fileName     name of the file.
     *
     * @return          the absolute path to the downloaded image file on the file system.
     */
    private static Uri createDirectoryAndSaveFile(Context context,
                                                  Bitmap image,
                                                  String fileName) {
        Utils.showLog(TAG, "fileName "+ fileName );
        File directory =
                new File(Environment.getExternalStoragePublicDirectory
                        (Environment.DIRECTORY_PICTURES)
                        + "/_miPhotoFiltered");

        if (!directory.exists()) {
            File newDirectory =
                    new File(directory.getAbsolutePath());
            newDirectory.mkdirs();
        }

        File file = new File(directory,
                getTemporaryFilename(fileName));
        if (file.exists())
            file.delete();

        try  {
            FileOutputStream outputStream = new FileOutputStream(file);
            image.compress(Bitmap.CompressFormat.JPEG,100, outputStream);
            outputStream.flush();
        } catch (Exception e) {
            e.printStackTrace();
            // Indicate a failure.
            return null;
        }

        // Get the absolute path of the image.
        String absolutePathToImage = file.getAbsolutePath();

        // Provide metadata so the downloaded image is viewable in the
        // Gallery.
        ContentValues values =
                new ContentValues();
        values.put(Images.Media.TITLE,
                fileName);
        values.put(Images.Media.DESCRIPTION,
                fileName);
        values.put(Images.Media.DATE_TAKEN,
                System.currentTimeMillis ());
        values.put(Images.ImageColumns.BUCKET_DISPLAY_NAME,
                file.getName().toLowerCase(Locale.US));
        values.put("_data",
                absolutePathToImage);

        ContentResolver cr =
                context.getContentResolver();

        // Store the metadata for the image into the Gallery.
        cr.insert(Images.Media.EXTERNAL_CONTENT_URI,
                values);

        Log.d(TAG,
                "absolute path to image file is "
                        + absolutePathToImage);

        return Uri.parse(absolutePathToImage);
    }
      /**
     * Create a temporary filename to store the result of a download.
     * 
     * @param url Name of the URL.
     * @return String containing the temporary filename.
     */
    static private String getTemporaryFilename(final String url) {
        // This is what you'd normally call to get a unique temporary
        // filename, but for testing purposes we always name the file
        // the same to avoid filling up student phones with numerous
        // files!
        //
        //REAL
         return Base64.encodeToString(url.getBytes(),
                                      Base64.NO_WRAP)
                                      + System.currentTimeMillis()+ ".jpg";

        //ESCRIBE ENCIMA
       // return Base64.encodeToString(url.getBytes(),Base64.NO_WRAP) + ".jpg";
    }

}
