package com.jesusrojo.miphoto.view;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.gesture.Gesture;
import android.gesture.GestureLibraries;
import android.gesture.GestureLibrary;
import android.gesture.GestureOverlayView;
import android.gesture.Prediction;
import android.os.Build;
import android.os.Bundle;

import android.support.annotation.NonNull;
import android.support.v7.app.AppCompatActivity;
import android.text.TextUtils;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.ImageView;

import com.google.android.gms.ads.AdRequest;
import com.google.android.gms.ads.AdView;
import com.google.android.gms.analytics.GoogleAnalytics;
import com.google.android.gms.analytics.Tracker;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GooglePlayServicesUtil;
import com.jesusrojo.miphoto.R;
import com.jesusrojo.miphoto.model.almacen.PhotosSQLite;
import com.jesusrojo.miphoto.model.almacen.IAlmacen;
import com.jesusrojo.miphoto.utils.Constants;
import com.jesusrojo.miphoto.utils.MyMediaPlayer;
import com.jesusrojo.miphoto.utils.Utils;
import com.jesusrojo.miphoto.view.ui.preferences.PreferenciasPreferenceActivity;
import com.jesusrojo.miphoto.view.ui.preferences.PreferenciasPreferenceFragment;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;


public class MainActivity  extends AppCompatActivity
        implements GestureOverlayView.OnGesturePerformedListener {

    private final String TAG = getClass().getSimpleName();
    private Activity mActivity;
    private Context mContext;
    private SharedPreferences mSharedPref;
    private SharedPreferences.Editor mEditor;

    private AdView adView;

    private GestureLibrary mGestureLibrary;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main_layout);
        Utils.showLog(TAG, "onCreate App");
        this.mActivity = this;
        this.mContext = getApplicationContext();

        //SHARED PREFERENCES
        mSharedPref = getSharedPreferences(Constants.PREFERENCES_DEFAULT, Context.MODE_PRIVATE);
        boolean credencials_status = mSharedPref.getBoolean("credencials_status", false);
        if (!credencials_status) {
            putDefaultInPreferences();
        }
        //DATOS DE CONEXION MIOS
        String hosting = mSharedPref.getString("hosting", "");
        if (TextUtils.isEmpty(hosting)) {
            putMyHostingInPreferences();
        }
        //ANALITICS
        Tracker mTracker =
                GoogleAnalytics.getInstance(getApplicationContext()).newTracker(R.xml.analytics);

        //PUBLICIDAD
        adView = (AdView) findViewById(R.id.adView);
        AdRequest adRequest = new AdRequest.Builder()
                .build();
        adView.loadAd(adRequest);

        comprobarGooglePlayServices();

        //ANIMACION
        if (mSharedPref.getBoolean("animacion_inicial", true)) {
            crearAnimacionInicial(this);
        }
        //MUSICA INICIAL
        if (mSharedPref.getBoolean("musica_inicial", true)) {
            MyMediaPlayer.crearMediaPlayer(mContext, R.raw.sound_camara1);
        }

        //GESTURES
        if (mSharedPref.getBoolean("gesture", true)) {
            mGestureLibrary = GestureLibraries.fromRawResource(this, R.raw.gestures);
            if (!mGestureLibrary.load()) {
                finish();
            }
            // crea la "capa" sobre la que se escriben los gestos descritos en
            // "gestures" y añade a esta mActivity
            GestureOverlayView mGestureOverlayView = (GestureOverlayView) findViewById(R.id.gestures);
            mGestureOverlayView.addOnGesturePerformedListener(this);
        }

    }//onCreate

    public void putMyHostingInPreferences() {

        String hosting = getResources().getString(R.string.hosting_url);
        String register = getResources().getString(R.string.register_php);
        String login = getResources().getString(R.string.login_php);
        String upload = getResources().getString(R.string.upload_php);
        String upload_binary = getResources().getString(R.string.upload_binary_php);
        String get_list = getResources().getString(R.string.get_list_php);

        mEditor = mSharedPref.edit();
        mEditor.putString("hosting", hosting);
        mEditor.putString("register", register);
        mEditor.putString("login", login);
        mEditor.putString("upload", upload);
        mEditor.putString("upload_binary", upload_binary);
        mEditor.putString("get_list", get_list);
        mEditor.apply();
    }

    //ON CLICK
    public void enter(View view) {
        startActivity(new Intent(this, PhotosListActivity.class));
        finish();
    }

    public void login(View view) {
        startActivity(new Intent(this, LoginActivity.class));
        finish();
    }

    //ANIMACION
    public void crearAnimacionInicial(Activity a) {
        //Views for the animation
        //  Button logo_main = (Button) a.findViewById(R.id.btn_logo_enter);
        ImageView logo_main = (ImageView) a.findViewById(R.id.iv_logo_main);
        Animation animacion =
                AnimationUtils.loadAnimation(this, R.anim.aparecer);
        logo_main.startAnimation(animacion);
    }

    //GESTURES
    @Override
    public void onGesturePerformed(GestureOverlayView overlay, Gesture gesture) {
        ArrayList<Prediction> prediction = mGestureLibrary.recognize(gesture);
        if (prediction.size() > 0) {
            String comando = prediction.get(0).name;

            if (comando.equalsIgnoreCase("play")) {
                login(null);
            } else if (comando.equalsIgnoreCase("configurar")) {
                lanzarSharedPreferences();
            } else if (comando.equalsIgnoreCase("acerca_de")) {
                lanzarInfoActivity();
            } else if (comando.equalsIgnoreCase("cancelar")) {
                finish();
            }
        }
    }

    //SHARED PREFERENCES
    public void lanzarSharedPreferences() {

        if (Build.VERSION.SDK_INT < 11) {
            startActivity(new Intent(mContext, PreferenciasPreferenceActivity.class));
        } else {
            startActivity(new Intent(mContext, PreferenciasPreferenceFragment.class));
        }
    }

    public void lanzarInfoActivity() {
        startActivity(new Intent(this, InfoActivity.class));
    }

    //SALVAR ESTADOS
    @Override
    protected void onSaveInstanceState(@NonNull Bundle estado) {
        super.onSaveInstanceState(estado);
        if (mSharedPref.getBoolean("musica_inicial", true)) {
            if (MyMediaPlayer.mediaPlayer != null) {
                int posicion = MyMediaPlayer.getPosicion();
                estado.putInt("posicion", posicion);
            }
        }
    }

    @Override
    protected void onRestoreInstanceState(@NonNull Bundle estado) {
        super.onRestoreInstanceState(estado);
        if (mSharedPref.getBoolean("musica_inicial", true)) {
            if (MyMediaPlayer.mediaPlayer != null) {
                int posicion = estado.getInt("posicion");
                MyMediaPlayer.putInPosicion(posicion);
            }
        }
    }

    //end of salvar estados
    //CICLOS ACTIVITY
    @Override
    public void onStart() {
        super.onStart();
        GoogleAnalytics.getInstance(this).reportActivityStart(this);
    }

    @Override
    protected void onRestart() {
        super.onRestart();
    }

    @Override
    public void onResume() {
        super.onResume();

        //LOGIN STATUS PREFERENCE
        boolean login_status = mSharedPref.getBoolean("login_status", false);
        String name = mSharedPref.getString("name", "");
        String table_name = mSharedPref.getString("table_name", "");
        String clean_table_name = table_name.substring(0, table_name.length() - 8);
        if (login_status) {
            //logueado
            getSupportActionBar().setTitle(clean_table_name);
        } else {
            putDefaultInPreferences();
        }
        Utils.showLog(TAG, "ON RESUME login_status: " + login_status + " name: " + name);

        //MUSICA INICIAL
        if (mSharedPref.getBoolean("musica_inicial", true)) {
            MyMediaPlayer.startMediaPlayer();
        }
        if (adView != null) {
            adView.resume();
        }
    }

    private void putDefaultInPreferences() {
        mEditor = mSharedPref.edit();
        mEditor.putString("name", "Default");
        mEditor.putString("password", "");
        mEditor.putBoolean("login_status", false);
        mEditor.putString("table_name", "Default_miPhoto");
        mEditor.apply();
        getSupportActionBar().setTitle("miPhoto");
    }

    @Override
    public void onPause() {
        //MUSICA INICIAL
        if (mSharedPref.getBoolean("musica_inicial", true)) {
            MyMediaPlayer.stopMediaPlayer();
        }
        if (adView != null) {
            adView.pause();
        }
        super.onPause();
    }

    @Override
    public void onStop() {
        GoogleAnalytics.getInstance(this).reportActivityStop(this);
        super.onStop();
    }

    @Override
    public void onDestroy() {
        if (mSharedPref.getBoolean("musica_inicial", true)) {
            MyMediaPlayer.releaseMediaPlayer();
        }
        if (adView != null) {
            adView.destroy();
        }
        Utils.showLog(TAG, "SEEK onDestroy()");
        super.onDestroy();
    }
    //fin ciclos Activity

    //MENU ANDROID
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent mActivity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.menu_item_preferences) {

            lanzarSharedPreferences();

            return true;
        }
        if (id == R.id.menu_item_reminder) {
            startActivity(new Intent(mContext, ReminderActivity.class));
            return true;
        }
        if (id == R.id.menu_item_datos_conexion) {
            startActivity(new Intent(mContext, DatosConexionActivity.class));
            return true;
        }
        if (id == R.id.menu_item_info) {
            startActivity(new Intent(mContext, InfoActivity.class));
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    //end menu Android
    private void comprobarGooglePlayServices() {
        // int status = GooglePlayServicesUtil.isGooglePlayServicesAvailable(this);
        int status = GooglePlayServicesUtil.isGooglePlayServicesAvailable(getBaseContext());

        if (status == ConnectionResult.SUCCESS) {

            //  Utils.showLog(TAG, "GooglePlayServices: SUCCESS");

        } else if (status == ConnectionResult.SERVICE_MISSING) {

            Utils.showLog(TAG, "GooglePlayServices: SERVICE_MISSING");
            Utils.showToast(this, "GooglePlay is missing in your device.");

        } else if (status == ConnectionResult.SERVICE_VERSION_UPDATE_REQUIRED) {

            Utils.showLog(TAG, "GooglePlayServices: SERVICE_VERSION_UPDATE_REQUIRED");
            Utils.showToast(this, "GooglePlay needs to be update on your device.");

        } else if (status == ConnectionResult.SERVICE_DISABLED) {

            Utils.showLog(TAG, "GooglePlayServices: SERVICE_DISABLED");
            Utils.showToast(this, "GooglePlay is disabled");

        } else {

            Utils.showLog(TAG, "STATUS: Error with GooglePlayServices");
            Utils.showToast(this, "Error with GooglePlayServices");
        }

        if (GooglePlayServicesUtil.isUserRecoverableError(status)) {
            ////NO PONER ALERT===  Dialog dialog = GooglePlayServicesUtil.getErrorDialog(status, this, 1);dialog.show();
            // Utils.showLog(TAG, "STATUS getError:" +GooglePlayServicesUtil.getErrorDialog(status, this, 111));
            //  Utils.showLog(TAG, "GooglePlayServices:" + status);
        }
    }
}
