package com.jesusrojo.miphoto.view;

import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.v4.app.FragmentActivity;
import android.view.View;
import android.widget.Toast;

import com.google.android.gms.ads.AdRequest;
import com.google.android.gms.ads.AdView;
import com.google.android.gms.analytics.GoogleAnalytics;
import com.google.android.gms.analytics.Tracker;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.MarkerOptions;
import com.jesusrojo.miphoto.R;
import com.jesusrojo.miphoto.utils.Utils;


public class MapEditActivity extends FragmentActivity
                 implements GoogleMap.OnMapClickListener {

    protected  final String TAG = getClass().getSimpleName();
    private Context mContext;

    private GoogleMap mMapa;
    private  LatLng mActualLatLng;

    private double mLatitud = 0;
    private double mLongitud = 0;

    public static boolean hasMarker;

    private AdView adView;
    public static boolean hasNewGeopunto;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.mapa_edit_layout);
        Utils.showLog(TAG, "onCreate");
        mContext = getApplicationContext();
        hasNewGeopunto = false;
        hasMarker = false;

        adView = (AdView)findViewById(R.id.adView);
        AdRequest adRequest = new AdRequest.Builder()
                .build();
        adView.loadAd(adRequest);

        //fragment map in XML
        mMapa = ((SupportMapFragment) getSupportFragmentManager()
                .findFragmentById(R.id.map)).getMap();

        mMapa.setMapType(GoogleMap.MAP_TYPE_NORMAL);
        mMapa.setMyLocationEnabled(true);
        mMapa.getUiSettings().setZoomControlsEnabled(true);
        mMapa.getUiSettings().setCompassEnabled(true);

        mMapa.setOnMapClickListener(this);
        Utils.showToastCustom(mContext, mContext.getResources().getString(R.string.click_map_and_save), "verde", 0);


    }//onCreate

    public boolean getPosicionActual() {

        if ( mMapa.getMyLocation() != null )  {

            mActualLatLng = new LatLng(mMapa.getMyLocation().getLatitude(),
                    mMapa.getMyLocation().getLongitude());

            Utils.showLog(TAG, "Posición actual: " + mActualLatLng.toString());
            return  true;
        }else{
            Utils.showLog(TAG, "Posicion actual desconocida," +
                    " activa GoogleMaps in your device");
            return false;
        }
    }
    public void animateCameraToMyActualPosition(View view) {
        if (getPosicionActual()) {
            mMapa.animateCamera(CameraUpdateFactory.newLatLngZoom(mActualLatLng, 12));
        }else{
            Utils.showToast(this, "Posicion actual desconocida," +
                    " activa GoogleMaps in your device");
        }
    }
    public void addMarkerCentral(View view) {
        mLatitud =mMapa.getCameraPosition().target.latitude;
        mLongitud =mMapa.getCameraPosition().target.longitude;

        mMapa.addMarker(new MarkerOptions().position(
                        new LatLng(mLatitud, mLongitud)));

        Utils.showLog(TAG, "Marker central:  mLatitud: " + mLatitud + " mLongitud: " + mLongitud);
        Utils.showToast(this, "Marker central:  mLatitud: " + mLatitud + " mLongitud: " + mLongitud);

    }
    public void cerrarYSalvarGeopunto(View view) {
        cerrarYGuardarGeopunto();
    }

    private void cerrarYGuardarGeopunto() {
        Intent i = new Intent();
        i.putExtra("path", "");
        i.putExtra("latitud", mLatitud);
        i.putExtra("longitud", mLongitud);
        setResult(RESULT_OK, i);
        if (mLatitud == 0 && mLongitud == 0){
            Utils.showToastCustom(this,
                    getResources().getString(R.string.you_did_not_select_a_place),"rojo",0);
            hasNewGeopunto = false;
        } else {
            hasNewGeopunto= true;
        }
        finish();
    }

    @Override
    public void onMapClick(LatLng puntoPulsado) {
        MarkerOptions marker1 = new MarkerOptions();
        marker1.position(puntoPulsado);
        marker1.icon(BitmapDescriptorFactory
                .defaultMarker(BitmapDescriptorFactory.HUE_YELLOW));
        if(!hasMarker) {
            mMapa.clear();
            mMapa.addMarker(marker1);
            hasMarker = true;
        } else {
            mMapa.clear();
            mMapa.addMarker(marker1);
            hasMarker =false;
        }
        mLatitud = puntoPulsado.latitude;
        mLongitud = puntoPulsado.longitude;
        Utils.showToastCustom(this, getResources().getString(R.string.do_not_forget_save_position), "rojo", 0);
    }

    //SALVAR ESTADOS
    @Override
    protected void onSaveInstanceState(@NonNull Bundle guardarEstado) {
        super.onSaveInstanceState(guardarEstado);
        guardarEstado.putDouble("param_latitud", mLatitud);
        guardarEstado.putDouble("param_longitud", mLongitud);
    }
    @Override
    protected void onRestoreInstanceState(@NonNull Bundle recEstado) {
        super.onRestoreInstanceState(recEstado);
        mLatitud = recEstado.getDouble("param_latitud");
        mLongitud = recEstado.getDouble("param_longitud");
    }
    /****** CICLOS ACTIVITY ******/
    @Override
    public void onStart() {
        super.onStart();
        GoogleAnalytics.getInstance(this).reportActivityStart(this);
    }
    @Override
    protected void onRestart() {
        super.onRestart();
    }
    @Override
    public void onResume() {
        super.onResume();
        if (adView != null) {
            adView.resume();
        }
    }
    @Override
    public void onPause() {
        if (adView != null) {
            adView.pause();
        }
        super.onPause();
    }
    @Override
    public void onStop() {
        GoogleAnalytics.getInstance(this).reportActivityStop(this);
        super.onStop();
    }
    @Override
    public void onDestroy() {
        if (adView != null) {
            adView.destroy();
        }
        super.onDestroy();
    }
    //fin ciclos Activity

    @Override
    public void onBackPressed() {

        if(mLatitud == 0 && mLongitud == 0){
           // Utils.showToastCustom(mContext, mContext.getResources().getString(R.string.you_did_not_save), "rojo", 0);
        }
        super.onBackPressed();
    }
}
