package com.jesusrojo.miphoto.view;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.net.Uri;
import android.os.Bundle;
import android.provider.MediaStore;
import android.support.annotation.NonNull;
import android.support.v4.app.FragmentActivity;
import android.view.View;
import android.widget.Button;

import com.google.android.gms.ads.AdRequest;
import com.google.android.gms.ads.AdView;
import com.google.android.gms.analytics.GoogleAnalytics;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;

import com.google.android.gms.maps.model.MarkerOptions;
import com.jesusrojo.miphoto.R;
import com.jesusrojo.miphoto.model.pojo.Photo;
import com.jesusrojo.miphoto.presenter.GetListTask;
import com.jesusrojo.miphoto.presenter.ITaskCallback;
import com.jesusrojo.miphoto.utils.Utils;
import com.jesusrojo.miphoto.utils.UtilsFile;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.OutputStream;
import java.util.List;


public class MapShowActivity extends FragmentActivity
                 implements  GoogleMap.OnInfoWindowClickListener,
                             LocationListener, ITaskCallback {

    protected  final String TAG = getClass().getSimpleName();
    private Activity mActivity;
    private Context mContext;
    private ProgressDialog mProgressDialog;


    public static boolean isTaskWorking;

    //MAPA
    private GoogleMap mMapa;
    private boolean isNormalMap;
    private  LatLng mActualLatLng;
    private double mLatitud = 0;
    private double mLongitud = 0;
    private  LatLng mLatLngFirstInList;

    //POSICION
    private LocationManager mLocationManager;
    private Location mLocation;
    private static final long DOS_MINUTOS= 2 * 60 * 1000;
    private static final long TIEMPO_MIN_GPS = 20 * 1000 ;
    private static final long TIEMPO_MIN_NET = 10 * 1000 ;
    private static final long DISTANCIA_MIN = 5 ; // 5 metros

    //admob
    private AdView adView;

    private List<Photo> mListPhotos;
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.mapa_show_layout);
        Utils.showLog(TAG, "onCreate");

        this.mContext = getApplicationContext();
        this.mActivity = this;

        adView = (AdView)findViewById(R.id.adView);
        AdRequest adRequest = new AdRequest.Builder()
                .build();
        adView.loadAd(adRequest);


        isNormalMap = true;

        //MAPA fragment map in XML
        mMapa = ((SupportMapFragment) getSupportFragmentManager()
                .findFragmentById(R.id.map)).getMap();
        mMapa.setMapType(GoogleMap.MAP_TYPE_NORMAL);
        mMapa.getUiSettings().setZoomControlsEnabled(true);
        mMapa.getUiSettings().setCompassEnabled(true);
        mMapa.setOnInfoWindowClickListener(this);

        Utils.showToast(mContext, mContext.getResources().getString(R.string.for_more_info_click_icon));

        //DATA TASK
        GetListTask getListTask = new GetListTask(this,mContext,
                MapShowActivity.this, mProgressDialog);
        getListTask.execute();

        //keep states on  ProgressDialog
        if (savedInstanceState != null) {
            isTaskWorking = savedInstanceState.getBoolean("isTaskWorking");
        }
        mProgressDialog = new ProgressDialog(this);

        comprobarGeoposicion(mContext);

    }//onCreate

    @Override
    public void onInfoWindowClick(Marker marker) {
        List<Photo> list  = mListPhotos;
        for(int id = 0 ; id< list.size();id ++){

            if(String.valueOf(list.get(id).getId()).equals(marker.getSnippet())){
                //OPEN FULL VIEW *****************
                Intent fullSelfieIntent = new Intent(Intent.ACTION_VIEW);
                fullSelfieIntent.setDataAndType(
                        Uri.parse(list.get(id).getPath()),
                        "image/*");
                startActivity(fullSelfieIntent);
            }
        }
    }

    private void comprobarGeoposicion(Context context) {
        //LOCATION
        mLocationManager = (LocationManager) context.getSystemService(LOCATION_SERVICE);
        //GPS
        if(mLocationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)){
            Location location = mLocationManager.getLastKnownLocation(
                    LocationManager.GPS_PROVIDER);
            if(location != null){
                setMyLocation(location);
                //  Utils.showToast(this, "GPS actual location: " + mLatitud + ", " + mLongitud);
            } else {
                // Utils.showToast(this, "GPS try to find location...");
            }
            actualizaMejorLocaliz(location);
        } else {
            Utils.showToast(this, "GPS in not available");
        }
        //NET
        if(mLocationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER)){

            Location location = mLocationManager.getLastKnownLocation(
                    LocationManager.NETWORK_PROVIDER);
            if(location != null){
                setMyLocation(location);
                // Utils.showToast(this, "NET actual location: " + mLatitud + ", " + mLongitud);
            } else {
                // Utils.showToast(this, "NET try to find location...");
            }
            actualizaMejorLocaliz(location);
        } else {
            Utils.showToast(this, "NET in not available");
        }
        //end location
    }

    private void setMyLocation(Location location) {
        mLatitud = location.getLatitude();
        mLongitud = location.getLongitude();
        mActualLatLng =  new LatLng(mLatitud,mLongitud);
    }
    private void activarLocationManager() {
        if(mLocationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)){
            mLocationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER,
                    TIEMPO_MIN_GPS, DISTANCIA_MIN, this);
        }
        if(mLocationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER)){
            mLocationManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER,
                    TIEMPO_MIN_NET, DISTANCIA_MIN, this);
        }
    }
    private void actualizaMejorLocaliz(Location location) {
        if (location != null) {
            if (location != null && mLocation == null
                    ||  location.getAccuracy() < 2 * mLocation.getAccuracy()
                    || location.getTime() - mLocation.getTime() > DOS_MINUTOS
                    ) {

                mLocation = location;
                setMyLocation(location);
                //MOSTRAR DONDE ESTOY
               // mMapa.moveCamera(CameraUpdateFactory.newLatLngZoom( new LatLng(mLatitud,mLongitud), 10));
            }
        }
    }

    // METODOS DE LA INTERFAZ  LocationListener
    @Override
    public void onLocationChanged(Location location) {
        if (location != null) {
            Utils.showLog(TAG,"onLocationChanged(");
            mLocationManager.removeUpdates(this);
            actualizaMejorLocaliz(location);
        }
    }
    @Override
    public void onProviderEnabled(String proveedor) {
        if (proveedor != null) {
            Utils.showLog(TAG,"onProviderEnabled(proveedor: " + proveedor);
            activarLocationManager();
        }
    }
    @Override
    public void onProviderDisabled(String proveedor) {
        Utils.showLog(TAG, "onProviderDisabled(proveedor:  " + proveedor);
        if (proveedor != null) {
            Utils.showLog(TAG, "onProviderDisabled(proveedor: " + proveedor);
            activarLocationManager();
        }
    }
    @Override
    public void onStatusChanged(String proveedor, int estado,
                                Bundle extras) {
          Utils.showLog(TAG, "Cambia estado proveedor: " + proveedor
                  + ", estado=" + estado + ", extras=" + extras);
          activarLocationManager();
    }
    //end metodos de la interfaz LocationListener

    //SALVAR ESTADOS
    @Override
    protected void onSaveInstanceState(@NonNull Bundle guardarEstado) {
        super.onSaveInstanceState(guardarEstado);
        guardarEstado.putDouble("latitud", mLatitud);
        guardarEstado.putDouble("longitud", mLongitud);
    }
    @Override
    protected void onRestoreInstanceState(@NonNull Bundle recEstado) {
        super.onRestoreInstanceState(recEstado);
        mLatitud = recEstado.getDouble("latitud");
        mLongitud = recEstado.getDouble("longitud");
    }
    //end of salvar estados

    //GOOGLE MAPS
    public boolean getPosicionActualFromMaps() {
        if ( mMapa.getMyLocation() != null )  {
            mActualLatLng = new LatLng(mMapa.getMyLocation().getLatitude(),
                    mMapa.getMyLocation().getLongitude());
            Utils.showLog(TAG, "GoogleMaps, posición actual: " + mActualLatLng.toString());
            return  true;
        }else{
            Utils.showLog(TAG, "GoogleMaps error, posición actual desconocida");
            return false;
        }
    }

    //ONCLICK
    public void animateCameraToMyActualPosition(View view) {
        mMapa.animateCamera(CameraUpdateFactory.newLatLngZoom(mActualLatLng, 15));
        mMapa.setMyLocationEnabled(true);//punto azul donde estoy

        /*from MAPS
        if (getPosicionActualFromMaps()) {
            mMapa.animateCamera(CameraUpdateFactory.newLatLngZoom(mActualLatLng, 15));
        }else{
            Utils.showToast(this, "GoogleMaps error, posición actual desconocida");
        }*/
    }
    public void onClickBtnSos(View view) {
        Intent intent = new Intent(Intent.ACTION_SEND);
        intent.setType("text/plain");
        intent.putExtra(Intent.EXTRA_TEXT, "SOS, I need help here : Lat: " + mLatitud + ", Long: " + mLongitud);
        startActivity(intent);
        Utils.showLog(TAG, "SOS: Lat: " + mLatitud + ", Long: " + mLongitud);
    }
    public void ponerSateliteONormal(View view) {
        Button btn  = (Button) findViewById(R.id.button_cambiarMapa);

        if (isNormalMap){
            btn.setText(getResources().getString(R.string.normal));
            mMapa.setMapType(GoogleMap.MAP_TYPE_SATELLITE);
            isNormalMap = false;
        } else {
            btn.setText(getResources().getString(R.string.satellite));
            mMapa.setMapType(GoogleMap.MAP_TYPE_NORMAL);
            isNormalMap= true;
        }
    }

    /****** CICLOS ACTIVITY ******/
    @Override
    public void onStart() {
        super.onStart();
        GoogleAnalytics.getInstance(this).reportActivityStart(this);
    }
    @Override
    protected void onRestart() {
        super.onRestart();
    }
    @Override
    public void onResume() {
        super.onResume();
        Utils.showLog(TAG, "onResume()");
        activarLocationManager();

        if (adView != null) {
            adView.resume();
        }
    }
    @Override
    public void onPause() {
        Utils.showLog(TAG, "onPause()");
        mLocationManager.removeUpdates(this);

        if (adView != null) {
            adView.pause();
        }
        super.onPause();
    }
    @Override
    public void onStop() {
        GoogleAnalytics.getInstance(this).reportActivityStop(this);
        super.onStop();
    }
    @Override
    public void onDestroy() {
        /**/
        if (adView != null) {
            adView.destroy();
        }
        super.onDestroy();
    }
    //fin ciclos Activity

    public void onClickSendMapa(View v) {
        Utils.showLog(TAG, "onClickSendMapa");
        captureScreen();
    }
    public void captureScreen() {
        //http://stackoverflow.com/questions/13773658/capture-screen-shot-of-googlemap-android-api-v2
        GoogleMap.SnapshotReadyCallback callback =
            new GoogleMap.SnapshotReadyCallback() {
                Bitmap bitmap = null;
                        @Override
                public void onSnapshotReady(Bitmap snapshot) {
                    bitmap = snapshot;
                            OutputStream fout = null;
                      String filePath = System.currentTimeMillis() + ".jpeg";
                    try {
                        fout = openFileOutput(filePath, MODE_WORLD_READABLE);
                         // Write the string to the file
                        bitmap.compress(Bitmap.CompressFormat.JPEG, 90, fout);
                        fout.flush();
                        fout.close();
                    } catch (FileNotFoundException e) {
                        Utils.showLogError(TAG, "Error e " + e);
                        filePath = "";
                    } catch (IOException e) {
                        Utils.showLogError(TAG, "Error e " + e);
                        filePath = "";
                    }
                    openShareImageDialog(filePath);
                }
            };
        mMapa.snapshot(callback);

    }
    public void openShareImageDialog(String filePath) {
        File file = this.getFileStreamPath(filePath);
        if(!filePath.equals("")) {
            final ContentValues values = new ContentValues(2);
            values.put(MediaStore.Images.Media.MIME_TYPE, "image/jpeg");
            values.put(MediaStore.Images.Media.DATA, file.getAbsolutePath());
            final Uri contentUriFile = getContentResolver().insert(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, values);

            final Intent intent = new Intent(android.content.Intent.ACTION_SEND);
            intent.setType("image/jpeg");
            intent.putExtra(android.content.Intent.EXTRA_STREAM, contentUriFile);
            startActivity(Intent.createChooser(intent, "Share Image"));
        }else{
            Utils.showLogError(TAG,"image share failed");
        }
    }

    @Override
    public void onFinishGetListTask(List<Photo> result) {
        //DATA
        mListPhotos = result;

        if (mListPhotos.size() > 0) {
            Double lat = mListPhotos.get(0).getLatitude();
            Double lng = mListPhotos.get(0).getLongitude();
            if (lat != 0 && lng != 0) {
                //centramos el ultimo lugar de la lista
                mLatLngFirstInList = new LatLng(lat, lng);
                mMapa.moveCamera(CameraUpdateFactory.newLatLngZoom(
                        mLatLngFirstInList, 15));
            }

            //CANVAS para la imagen
            Bitmap.Config conf = Bitmap.Config.ARGB_8888;
            Bitmap bmp = Bitmap.createBitmap(80, 80, conf);
            Canvas canvas1 = new Canvas(bmp);
            // paint defines the text color,
            // stroke width, size
            Paint color = new Paint();
            color.setTextSize(35);
            color.setColor(Color.BLACK);

            // mostramos  la lista
            for (Photo photo : mListPhotos) {
                String title = "open";
                long id = photo.getId();

                lat = photo.getLatitude();
                lng = photo.getLongitude();

                //modify canvas
                canvas1.drawBitmap(
                        UtilsFile.getThumbnail(
                                photo.getPath().substring(8)), 0, 0, color);
                //canvas1.drawText("User Name!", 30, 40, color);

                if (lat != 0 && lng != 0) {
                    //añadimos marcador con titulo
                    mMapa.addMarker(new MarkerOptions()
                                    .position(new LatLng(lat, lng))
                                    .title(title)
                                    .snippet(String.valueOf(id))
                                    .icon(BitmapDescriptorFactory.fromBitmap(bmp))
                    );
                }
            }//for
        }//if DATA
        ///end DATA
    }
    ///////////////////////////////////////////////////////null
    @Override
    public void onFinishDeletePojoTask(String result) {}
    @Override
    public void onFinishUpdatePathFilterTask(String result) {}
    @Override
    public void onFinishSavePojoNubeTask(String result, Photo photo) {}
}
