package com.jesusrojo.miphoto.view;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.provider.MediaStore;
import android.support.v4.app.DialogFragment;
import android.support.v4.content.LocalBroadcastManager;
import android.support.v7.app.AppCompatActivity;
import android.text.TextUtils;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;

import com.google.android.gms.ads.AdRequest;
import com.google.android.gms.ads.AdView;
import com.google.android.gms.analytics.GoogleAnalytics;
import com.google.android.gms.analytics.Tracker;
import com.jesusrojo.miphoto.R;
import com.jesusrojo.miphoto.model.almacen.IAlmacen;
import com.jesusrojo.miphoto.model.almacen.PhotosSQLite;
import com.jesusrojo.miphoto.model.pojo.Photo;
import com.jesusrojo.miphoto.presenter.DeletePojoTask;
import com.jesusrojo.miphoto.presenter.GetListTask;
import com.jesusrojo.miphoto.presenter.ITaskCallback;
import com.jesusrojo.miphoto.presenter.SavePojoNubeTask;
import com.jesusrojo.miphoto.presenter.UpdatePathFilterTask;
import com.jesusrojo.miphoto.presenter.UpdateStatusTask;
import com.jesusrojo.miphoto.presenter.services.DeleteBinaryIntentService;
import com.jesusrojo.miphoto.presenter.services.FilterIntentService;
import com.jesusrojo.miphoto.presenter.services.UploadPhotoIntentService;
import com.jesusrojo.miphoto.utils.Constants;
import com.jesusrojo.miphoto.utils.Utils;
import com.jesusrojo.miphoto.utils.UtilsFile;
import com.jesusrojo.miphoto.view.ui.PhotosBaseAdapter;
import com.jesusrojo.miphoto.view.ui.dialogFragment.DeleteDialog;
import com.jesusrojo.miphoto.view.ui.dialogFragment.IDialogListener;

import java.io.File;
import java.util.List;


public class PhotosListActivity extends //FragmentActivity
                                             AppCompatActivity
        implements AdapterView.OnItemClickListener,
                   AdapterView.OnItemLongClickListener,
                   ITaskCallback, IDialogListener {

    protected  final String TAG = getClass().getSimpleName() ;
    private static Activity mActivity;
    private static Context mContext;
    private SharedPreferences mSharedPref;
    private SharedPreferences.Editor editor;

    public static IAlmacen mIAlmacen;
    public static String folder;
    public static String folder_filter;


    public static PhotosBaseAdapter mBaseAdapter;

    private final static int REQUEST_CAMERA = 1;
    private final static int REQUEST_GALLERY = 3;
    private File mFile;
    public static Uri mImageUri;

    private ProgressDialog mProgressDialog;
    public static boolean isTaskWorking;
    private ListView listView;
    boolean isCheckAll;


    private UploadResultReceiver mUploadResultReceiver;
    private FilterResultReceiver mFilterResultReceiver;

    private  List<Photo> mListPhotos;
    private String[] mSelectArgs;
    private Photo mPhoto;

    private DeleteBinaryResultReceiver mDeleteBinaryResultReceiver;

    private AdView adView;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.photos_list_layout);
        mActivity = this;
        mContext = getApplicationContext();

        //PUBLICIDAD
        adView = (AdView)findViewById(R.id.adView);
        AdRequest adRequest = new AdRequest.Builder()
                .build();
        adView.loadAd(adRequest);

        //SHARED PREFERENCES
        mSharedPref = getSharedPreferences(Constants.PREFERENCES_DEFAULT, Context.MODE_PRIVATE);
        //title of activity with AppCompactActivity
        String table_name = mSharedPref.getString("table_name", "");
        String clean_table_name = table_name.substring(0, table_name.length() - 8);
        getSupportActionBar().setTitle(clean_table_name);

        //folders where we keep data in the device
        folder = table_name ;
        folder_filter = table_name + "Filtered" ;

        //ALMACEN
        mIAlmacen = new PhotosSQLite(mContext, table_name);

        //ADAPTER
        mBaseAdapter = new PhotosBaseAdapter(this);

        //keep states on  ProgressDialog
        if (savedInstanceState != null) {
            isTaskWorking = savedInstanceState.getBoolean("isTaskWorking");
        }
        mProgressDialog = new ProgressDialog(this);

        //DATA con AsynTask
        empezarGetListTask();
        //DATA sin AsyncTask
        //List<Photo> list = new ArrayList<>();
        //list= MainActivity.mIAlmacen.getListPhotos(getApplicationContext());
        //mBaseAdapter.setListaSelfies(list);
        //LISTA
        listView = (ListView) findViewById(R.id.list);
        listView.setAdapter(mBaseAdapter);
        listView.setOnItemClickListener(this);
        listView.setOnItemLongClickListener(this);

        isCheckAll = true;

          // Receiver for the notification for upload photo
        mUploadResultReceiver = new UploadResultReceiver();

        // Receiver for the notification for delete photo
        mDeleteBinaryResultReceiver = new DeleteBinaryResultReceiver();

        // Receiver for the notification after filter photo
        mFilterResultReceiver = new FilterResultReceiver();

    }//onCreate

    @Override
    protected void onActivityResult(int requestCode, int resultCode,
                                    Intent data) {
        if (resultCode == RESULT_OK
                && requestCode == REQUEST_CAMERA) {
            Utils.showLog(TAG, "CAMERA mImageUri: " + mImageUri);

            UtilsFile.galleryAddPic(mActivity, String.valueOf(mImageUri));
            //***********************************************
           // mPhoto = new Photo(null, mName, mImageUri,"",mLatitude, mLongitude,"", System.currentTimeMillis());
          //  startSavePojoTask(mActivity,mContext,mPhoto);
            //***********************************************
            startActivity(new Intent(this, PhotoSaveEditActivity.class)
                    .putExtra("job", "job_save")
                    .putExtra("path", mImageUri.toString()));
            finish();
        }
        if ( resultCode == Activity.RESULT_OK
                && requestCode == REQUEST_GALLERY) {
            if (data != null) {
                mImageUri = data.getData();
                Utils.showLog(TAG, "GALLERY mImageUri: " + mImageUri);
                startActivity(new Intent(this, PhotoSaveEditActivity.class)
                        .putExtra("job", "job_copy")
                        .putExtra("path", mImageUri.toString()));
                finish();
            }
        }
    }
    public void onClickCamera(View v) {
        makeIntentCamera();
    }
    public void onClickGallery(View view) { makeIntentGallery();}
    public void onClickMap(View view) {startActivity(new Intent(this, MapShowActivity.class));}
    public void onClickShare(View view) { prepararIntentShare(); }

    private void makeIntentCamera() {
        Intent intent = new Intent("android.media.action.IMAGE_CAPTURE");
      //  String folder = mSharedPref.getString("table_name", "");
        try {
            mFile = UtilsFile.createFileInMyFolder(mContext,
                    PhotosListActivity.folder, null);
        } catch (Exception e) {
            Utils.showLogError(TAG, "Error e: " + e);
        }
        if( mFile != null ) {
            mImageUri = Uri.fromFile(mFile);
            Utils.showLog(TAG, "mImageUri: " + mImageUri);

            //check that the create intent is not null
            if (intent.resolveActivity(getPackageManager()) != null) {

                intent.putExtra(MediaStore.EXTRA_OUTPUT, mImageUri);
                startActivityForResult(intent, REQUEST_CAMERA);
            }
        } else {
            Utils.showToastCustom(mContext, mActivity.getResources().getString(R.string.error_file_not_exits), "rojo", 0);
        }
    }
    private void makeIntentGallery() {
        // http://stackoverflow.com/questions/19837358/android-kitkat-securityexception-when-trying-to-read-from-mediastore
        // i = new Intent(Intent.ACTION_PICK,
        //        android.provider.MediaStore.Images.Media.EXTERNAL_CONTENT_URI);
        //         startActivityForResult(i, CHOOSE_IMAGE);

        if (  Build.VERSION.SDK_INT < 19 ) {
            Intent intent = new Intent();
            intent.setType("image/*");
            intent.setAction(Intent.ACTION_GET_CONTENT);
            startActivityForResult(Intent.createChooser(intent,
                    "Select Picture"), REQUEST_GALLERY);
        } else {
            // ACTION_OPEN_DOCUMENT is the intent to choose a file via the system's file
            // browser.
            Intent intent = new Intent(Intent.ACTION_OPEN_DOCUMENT);
            // Filter to only show results that can be "opened", such as a
            // file (as opposed to a list of contacts or timezones)
            intent.addCategory(Intent.CATEGORY_OPENABLE);
            // Filter to show only images, using the image MIME data type.
            // If one wanted to search for ogg vorbis files, the type would be "audio/ogg".
            // To search for all documents available via installed storage providers,
            // it would be "*/*".
            intent.setType("image/*");
            startActivityForResult(intent, REQUEST_GALLERY);
        }
    }
    private void prepararIntentShare() {
        //make a list with the photos selected
        if (getSelectArgs() != null) {
            String[] selectArgs = getSelectArgs();
            if (selectArgs.length > 0) {
                for (int i = 0; i < selectArgs.length; i++) {
                    //get the bitmap to share
                    Bitmap bitmap = null;
                    bitmap = UtilsFile.decodeSampledBitmapFromFile(mContext,
                                    selectArgs[i].substring(8), 1000, 700);
                    if (bitmap != null) {
                        //get the uri from bitmap
                        Uri uri = UtilsFile.getUriFromBitmap(mContext, bitmap);

                        if (uri != null) {
                            //intent para compartir la foto
                            makeIntentSharePhoto(uri);
                        } else {
                            Utils.showToastCustom(mContext, mActivity.getResources().getString(R.string.error_bitmap_not_exits), "rojo", 0);
                        }
                    } else {
                        Utils.showToastCustom(mContext, mActivity.getResources().getString(R.string.error_file_not_exits), "rojo", 0);
                    }
                }//for
            } else {
                Utils.showToastCustom(mContext, mActivity.getResources().getString(R.string.select_photo_s), "rojo", 0);
            }
        }
    }
    private void makeIntentSharePhoto(Uri uri) {
        Intent share = new Intent(Intent.ACTION_SEND);
        share.setType("image/jpeg");
        share.putExtra(Intent.EXTRA_STREAM, uri);
        startActivity(Intent.createChooser(share, "Share Image"));
    }

    private void edit() {
        String[] selectArgs = getSelectArgs();
        if(selectArgs != null) {
            if (selectArgs.length != 1) {
                Utils.showToastCustom(mContext, mActivity.getResources().getString(R.string.select_only_one), "rojo", 0);
            } else {
                if (selectArgs[0] != null) {
                    //intent to the SaveEditActivity with the select path
                    Intent intent = new Intent(PhotosListActivity.this,
                            PhotoSaveEditActivity.class);
                    intent.putExtra("path", selectArgs[0]);
                    intent.putExtra("job", "job_edit");
                    startActivity(intent);
                    finish();
                }
            }
        }
     }

    private void checkAll() {
        if (isCheckAll) {
            if(mBaseAdapter.getCount() > 0) {
                mBaseAdapter.checkAll(true);
                // btn_checkAll.setText(" Uncheck ");
                Utils.showToastCustom(mContext, mActivity.getResources().getString(R.string.all_selected), "verde", 0);
                isCheckAll = false;
            } else {
                Utils.showToastCustom(mContext, mActivity.getResources().getString(R.string.list_is_empty), "rojo", 0);
            }
        } else {
            // btn_checkAll.setText("Check all");
            Utils.showToastCustom(mContext,mActivity.getResources().getString( R.string.none_selected), "rojo", 0);
            mBaseAdapter.checkAll(false);
            isCheckAll = true;
        }
    }

    //lista de argumentos  seleccionados del adapter
    public List<Photo> getlistPhotosSelected() {
        List <Photo> list = mBaseAdapter.getListPhotosSelected();
        if(list == null || list.size() == 0){
            Utils.showToastCustom(mContext, mContext.getResources().getString(R.string.select_photo_s), "rojo", 0);
            return null;
        } else {
            mListPhotos = list;
            return list;
        }
    }
    public String [] getSelectArgs(){
        //make array String  with the photos  path selected
        List<Photo> list = getlistPhotosSelected();
        if(list == null) {
            return null;
        } else {
            String[] selectArgs;
            selectArgs = new String[list.size()];
            for (int i = 0; i <selectArgs.length ; i++) {
                selectArgs[i] = list.get(i).getPath();
                Utils.showLog(TAG, "selectArg[i]" + selectArgs[i]);
            }
            mSelectArgs = selectArgs;
            return selectArgs;
        }
    }
    private void delete() {
        if(getSelectArgs() != null) {
            DialogFragment newFragment = DeleteDialog.newInstance("job_delete");
            newFragment.show(getSupportFragmentManager(), "dialog");
        }
    }
    @Override
    public void eleccionDeleteBinaryIntentService(DialogFragment dialog) {
        mSelectArgs = getSelectArgs();
        if(mSelectArgs != null ) {
            if (mSelectArgs.length > 0) {
                for (int i = 0; i < mSelectArgs.length; i++) {
                    
                    deleteBinaryWithIntentService(mSelectArgs[i], i );
                }
            } else {
                Utils.showToastCustom(mContext, mActivity.getResources().getString(R.string.select_photo_s), "rojo", 0);
            }
        }
    }
    //DELETE SERVICE
    private void deleteBinaryWithIntentService(String selectPath, int index) {
        startService(DeleteBinaryIntentService.makeIntent(getApplicationContext(),
                Uri.parse(selectPath), index));
    }
    private class DeleteBinaryResultReceiver extends BroadcastReceiver {
        @Override
        public void onReceive(Context context, Intent intent) {

            String respuesta = intent.getStringExtra("respuesta");
            String path = intent.getStringExtra("path");

            if (respuesta.equals(Constants.SUCCESS_DELETED)) {

                String[] selectArgs = mSelectArgs;
                for (int i = 0; i < selectArgs.length ; i++) {
                    if(selectArgs[i].equals(path)) {
                        empezarDeletePojoTask(path);
                    }
                }
            } else {
                Utils.showToast(context, Constants.ERROR_NOT_DELETED);
            }

            /*
            List<Photo> list = mListPhotos;
            for (int i = 0; i < list.size() ; i++) {
                if(list.get(i).getPath().equals(path)) {
                    Photo photo = list.get(i);
                    SavePojoNubeTask task =
                            new SavePojoNubeTask(mActivity, mContext);
                    task.execute(photo);
                }
            }*/
        }
    }
    private void registerReceiverDeleteBinary() {
        // Create an Intent filter that handles Intents from the IntentService.
        IntentFilter intentFilter =
                new IntentFilter(DeleteBinaryIntentService.ACTION_DELETE_BYNARY_RESPONSE);
        intentFilter.addCategory(Intent.CATEGORY_DEFAULT);
        // Register the BroadcastReceiver.
        LocalBroadcastManager.getInstance(this).registerReceiver(mDeleteBinaryResultReceiver,
                intentFilter);
    }

    public void empezarDeletePojoTask(String path) {
        String[] selectArgs = new String[1];
        selectArgs[0] = path;
        DeletePojoTask task =
                new DeletePojoTask(mActivity, mContext, (ITaskCallback) mActivity);
        task.execute(selectArgs);
    }
    @Override
    public void onFinishDeletePojoTask(String result) {
            empezarGetListTask();
    }
    //
    private void empezarGetListTask() {
        GetListTask getListTask = new GetListTask(mActivity,mContext,this, mProgressDialog);
        getListTask.execute();
    }
    @Override
    public void onFinishGetListTask(List<Photo> result) {
      if(result.size() == 0){
          Utils.showToast(mContext, "The list is empty");
      }
      mBaseAdapter.setListaSelfies(result);
    }

    //UPLOAD
    private void uploadPhotos() {
        boolean login_device = mSharedPref.getBoolean("login_device", false );
        boolean login_status = mSharedPref.getBoolean("login_status", false );
        if ( !login_device ) {
        if ( login_status ) {
            if( DatosConexionActivity.checkDatosConexion(mSharedPref) ) {

                mListPhotos = getlistPhotosSelected();
                if(mListPhotos != null) {
                    for (int i = 0; i < mListPhotos.size(); i++) {
                        Photo photo = mListPhotos.get(i);
                        if (photo != null) {
                            Utils.showLog(TAG, "photo.getStatus() " +photo.getStatus());
                            if (photo.getStatus()==0 ) {

                                uploadPhotoWithIntentService(
                                        Uri.parse(mListPhotos.get(i).getPath()));

                            } else {
                                Utils.showToastCustom(mContext, mActivity.getResources().getString(R.string.photo_already_uploaded), "rojo", 0);
                            }
                        } else {
                            Utils.showToastCustom(mContext, mActivity.getResources().getString(R.string.error_file_not_exits), "rojo", 0);
                        }
                    }//for
                }
            } else {
                Utils.showToastCustom(mContext, mActivity.getResources().getString(R.string.no_hosting_data_conexion), "rojo", 0);
            }
        } else {
            Utils.showToastCustom(this, mActivity.getResources().getString(R.string.you_need_a_cloud_account), "rojo", 0);
        }
        } else {
            Utils.showToastCustom(this, mActivity.getResources().getString(R.string.you_need_a_cloud_account), "rojo", 0);
        }
    }

    public void uploadPhotoWithIntentService(Uri uri){
        // Sends an Intent command to the UploadPhotoIntentService.
       startService(UploadPhotoIntentService.makeIntent(getApplicationContext(), uri));
    }
    private class UploadResultReceiver extends BroadcastReceiver {
        private final String TAG = getClass().getSimpleName();
        @Override
        public void onReceive(Context context,
                              Intent intent) {
            String respuesta = intent.getStringExtra("respuesta");
            String path = intent.getStringExtra("path");

            if (respuesta.equals(Constants.SUCCESS_UPLOADED)) {
                    List<Photo> list = mListPhotos;
                    for (int i = 0; i < list.size() ; i++) {
                        if(list.get(i).getPath().equals(path)) {
                            Photo photo = list.get(i);
                            SavePojoNubeTask task =
                                    new SavePojoNubeTask(mActivity, mContext,
                                            PhotosListActivity.this);
                            task.execute(photo);
                        }
                    }
             } else {
                Utils.showToast(context, Constants.ERROR_NOT_UPLOADED);
            }
         }
    }
    private void registerReceiverUpload() {
        // Create an Intent filter that handles Intents from the IntentService.
        IntentFilter intentFilter =
                new IntentFilter(UploadPhotoIntentService.ACTION_UPLOAD_RESPONSE);
        intentFilter.addCategory(Intent.CATEGORY_DEFAULT);
        // Register the BroadcastReceiver.
        LocalBroadcastManager.getInstance(this).registerReceiver(mUploadResultReceiver,
                intentFilter);
    }

    @Override
    public void onFinishSavePojoNubeTask(String result,Photo photo) {
        String[] args = new String[2];
        args[0] = photo.getPath();
        args[1] = "1";
        UpdateStatusTask task = new UpdateStatusTask(mActivity,mContext,
                PhotosListActivity.this);
        task.execute(args);
       empezarGetListTask();
    }

    //GUARDAR ESTADOS
    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putBoolean("isTaskWorking", isTaskWorking);
    }
    @Override
    protected void onRestoreInstanceState(Bundle savedInstanceState) {
        super.onRestoreInstanceState(savedInstanceState);
        if (savedInstanceState.containsKey("isTaskWorking")) {
            isTaskWorking = savedInstanceState.getBoolean("isTaskWorking");
            if ( isTaskWorking ) {
                mProgressDialog.show();
            }
        }
    }

    //MENU ANDROID
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_photos_list, menu);
        return true;
    }
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch(item.getItemId()) {
            case R.id.accion_upload:
                uploadPhotos();
                return true;
            case R.id.accion_borrar:
                delete();
                return true;
            case R.id.accion_select_all:
                checkAll();
                return true;

            case R.id.accion_edit:
                edit();
                return true;

            case R.id.accion_filter:
                filterPhoto();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    //CICLOS ACTIVITY
    @Override
    public void onStart() {
        super.onStart();
        GoogleAnalytics.getInstance(this).reportActivityStart(this);
    }
    @Override
    protected void onRestart() {
        super.onRestart();
    }
    @Override
    protected void onResume() {
        super.onResume();
        // Register BroadcastReceiver that receives result from
        // IntentService when the job is done
        registerReceiverDeleteBinary();
        registerReceiverUpload();
        registerReceiverFilter();

        //admob
        if (adView != null) {
            adView.resume();
        }
    }
    @Override
    protected void onPause() {
        super.onPause();
        // Unregister BroadcastReceiver.
        LocalBroadcastManager.getInstance(this).unregisterReceiver(mUploadResultReceiver);
        LocalBroadcastManager.getInstance(this).unregisterReceiver(mDeleteBinaryResultReceiver);
        LocalBroadcastManager.getInstance(this).unregisterReceiver(mFilterResultReceiver);

        //admob
        if (adView != null) {
            adView.pause();
        }
    }
    @Override
    public void onStop() {
        GoogleAnalytics.getInstance(this).reportActivityStop(this);
        super.onStop();
    }
    @Override
    public void onDestroy() {
        if (adView != null) {
            adView.destroy();
        }
        super.onDestroy();
    }
    //fin ciclos Activity

    //FILTER
    public void filterPhoto() {
        mListPhotos = getlistPhotosSelected();
        if(mListPhotos != null) {
            for (int i = 0; i < mListPhotos.size(); i++) {
                Photo photo = mListPhotos.get(i);
                if (photo != null) {

                    int countFiltered = 1;
                    if (photo.getPathFilter().equals("")) {

                        empezarFilterIntentService(Uri.parse(mListPhotos.get(i).getPath()));

                    } else {

                        String resp = countFiltered++ +
                                mActivity.getResources().getString(R.string.photo_already_filtered);
                        Utils.showToastCustom(mContext, resp, "rojo", 0);
                    }
                } else {
                    Utils.showToastCustom(mContext, mActivity.getResources().getString(R.string.error_file_not_exits), "rojo", 0);
                }
            }//for
        }
    }
    /***********************************************************
     * The Broadcast Receiver that registers itself to receive result
     * from .
     ***************************************************************///////////////BROADCAST RECEIVER FILTER
    public void empezarFilterIntentService(Uri uri){
        // Sends an Intent command to the UploadPhotoIntentService.
        startService(FilterIntentService.makeIntent(getApplicationContext(), uri));
    }
    private class FilterResultReceiver extends BroadcastReceiver {
        @Override
        public void onReceive(Context context,
                              Intent intent) {
            String respuesta = intent.getStringExtra("respuesta");
            String path = intent.getStringExtra("path");
            String pathFiltered = intent.getStringExtra("pathFiltered");

            if (respuesta.equals(Constants.SUCCESS_FILTER)) {

                //create an array for the async task
                String[] dataForUpdate= new String[2];
                dataForUpdate[0]= path;
                dataForUpdate[1]= pathFiltered;
                    //update the pojo data with the path and the pathFiltered
                    UpdatePathFilterTask Task =
                            new UpdatePathFilterTask( mActivity,mContext,
                            PhotosListActivity.this);
                    Task.execute(dataForUpdate);
            } else {
                Utils.showToast(context, Constants.ERROR_NOT_FILTERED);
            }
        }
    }
    private void registerReceiverFilter() {
        IntentFilter intentFilter =
                new IntentFilter(FilterIntentService.ACTION_FILTER_RESPONSE);
        intentFilter.addCategory(Intent.CATEGORY_DEFAULT);
        // Register the BroadcastReceiver.
        LocalBroadcastManager.getInstance(this).registerReceiver(mFilterResultReceiver, intentFilter);
    }

    @Override
    public void onFinishUpdatePathFilterTask(String result) {
        empezarGetListTask();
    }

    @Override
    public void onItemClick(AdapterView<?> parent,View view, int position,long id) {
       //  Utils.showToast(this, "Position: " + position + " Id: " + id);
    }
    @Override
    public boolean onItemLongClick(AdapterView<?> parent,View view,
                                   int position,long id) {
        // Utils.showToast(this, "LONG CLICK Position: " + position + " ID: " + id);
        // return true;
        return false;
    }
    @Override
    public void onBackPressed() {
        String tempName = mSharedPref.getString("temp_name", "");
        if(!TextUtils.isEmpty(tempName)){
            //delete the temporal name,  and the login status
            // because the user dont want to keep logged
            editor = mSharedPref.edit();
            editor.putString("temp_name", "");
            editor.putBoolean("login_status", false);
            editor.apply();
         }
        super.onBackPressed();
    }
}



