package com.jesusrojo.miphoto.view;


import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.support.v7.app.AppCompatActivity;
import android.text.TextUtils;
import android.view.View;
import android.widget.EditText;

import com.google.android.gms.ads.AdRequest;
import com.google.android.gms.ads.AdView;
import com.google.android.gms.analytics.GoogleAnalytics;
import com.jesusrojo.miphoto.R;
import com.jesusrojo.miphoto.model.almacen.UsersSQLite;
import com.jesusrojo.miphoto.model.pojo.User;
import com.jesusrojo.miphoto.presenter.RegisterTask;
import com.jesusrojo.miphoto.utils.Constants;
import com.jesusrojo.miphoto.utils.Utils;
import com.jesusrojo.miphoto.view.ui.dialogFragment.TerminosDialog;

public class RegisterActivity extends AppCompatActivity
        implements RegisterTask.IRegisterTaskCallback {

    private final String TAG = getClass().getSimpleName();
    private Activity mActivity;
    private Context mContext;
	public static SharedPreferences mSharedPref;
    private SharedPreferences.Editor mEditor;

    private ProgressDialog progressDialog;
	public static boolean isTaskWorking;

	private String name , password , password2 , mail ;
	private EditText ET_name, ET_password, ET_password2, ET_mail;
    private AdView adView;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.register_layout);
        Utils.showLog(TAG, "onCreate");
        this.mActivity = this;
        this.mContext = getApplicationContext();

        //SHARED PREFERENCES
        mSharedPref = getSharedPreferences(Constants.PREFERENCES_DEFAULT, Context.MODE_PRIVATE);

        adView = (AdView)findViewById(R.id.adView);
        AdRequest adRequest = new AdRequest.Builder()
                .build();
        adView.loadAd(adRequest);
        //VIEWS
		ET_name = (EditText) findViewById(R.id.editText_user);
		ET_password = (EditText) findViewById(R.id.editText_password);
		ET_password2 = (EditText) findViewById(R.id.editText_password2);
		ET_mail = (EditText) findViewById(R.id.editText_mail);

        //keep states on  ProgressDialog
        if (savedInstanceState != null) {
            isTaskWorking = savedInstanceState.getBoolean("isTaskWorking");
        }
        progressDialog = new ProgressDialog(mActivity);

    }//onCreate
   //VALIDACIONES
    private boolean validarFormularioRegistro() {
        if(TextUtils.isEmpty(name) ){
            Utils.showToast(mContext, "Error: name is missing");
            return true;
        }
        //sin TextUtils
        if (password == null || password.length() == 0 ) {
            Utils.showToast(mContext, "Error, password is missing");
            return true;
        }
        if (TextUtils.isEmpty(mail)) {
            Utils.showToast(mContext, "Error: Email missing");
            return true;
        }
        if (!isValidEmail(mail)) {
            Utils.showToast(mContext, "Error: Email no valid");
            return true;
        }

        //extras
        if (name.length() < 5) {
            Utils.showToast(mContext, "Error: Name minimum length is 5");
            return true;
        }
        if (password.length() < 5) {
            Utils.showToast(mContext,"Error: Password minimum length is 5");
            return true;
        }
        if (password.compareTo(password2) != 0) {
            Utils.showToast(mContext, "Error, Password are not the same");
            return true;
        }
        return false;
    }
    public final static boolean isValidEmail(CharSequence target) {
        return !TextUtils.isEmpty(target)
                && android.util.Patterns.EMAIL_ADDRESS.matcher(target).matches();
    }
    private void getDatosDelFormulario() {
        name = ET_name.getText().toString();
        password = ET_password.getText().toString();
        password2 = ET_password2.getText().toString();
        mail = ET_mail.getText().toString();
    }
    public void showTerminosCondiciones(View view) {
        DialogFragment newFragment = TerminosDialog.newInstance(
                    R.string.terminos_condiciones,
                    R.string.terminos_condiciones_explicacion,
                    null);
        newFragment.show(getSupportFragmentManager(), "dialog");
    }
    public void makeIntentDatosConexion(View view) {
        startActivity(new Intent(this, DatosConexionActivity.class));
        // finish();
    }
    public void register(View v) {
        Utils.hideSoftKeyboard(RegisterActivity.this);
        if(DatosConexionActivity.checkDatosConexion(mSharedPref)) {

            getDatosDelFormulario();

            if (validarFormularioRegistro()) return;

            RegisterTask task = new RegisterTask(mActivity, mContext,
                                          progressDialog, RegisterActivity.this);
            switch(v.getId()) {
                case R.id.btn_register_device:

                    task.empezarTask(name, password, mail,"device");
                    isTaskWorking = true;
                    break;
                case R.id.btn_register_cloud:

                    task.empezarTask(name, password, mail, "cloud");
                    isTaskWorking = true;
                    break;
            }//switch
        } else {
            Utils.showToastCustom(mActivity, mActivity.getResources().getString(R.string.no_hosting_data_conexion), "rojo", 0);
        }
    }
    @Override
    public void onFinishMyRegisterTask(String result) {
        if ( result.equals(Constants.SUCCESS) ) {
            saveSharedPreferencess();
            makeIntentLoginActivity();
        } else {
            Utils.showToastCustom(mContext, result, "rojo", 0);
        }  
    }
    private void saveSharedPreferencess() {
        mEditor = mSharedPref.edit();
        mEditor.putString("name", name);
        mEditor.putString("password", password);
        mEditor.apply();
        Utils.showToast(mActivity, mActivity.getResources().getString(R.string.saved_credentials) + name + "  " + password);
    }
    private void makeIntentLoginActivity() {
        mActivity.startActivity(new Intent(mActivity, LoginActivity.class));
        mActivity.finish();
    }
    //GUARDAR ESTADOS
    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putBoolean("isTaskWorking", isTaskWorking);
    }
    @Override
    protected void onRestoreInstanceState(Bundle savedInstanceState) {
        super.onRestoreInstanceState(savedInstanceState);
        if (savedInstanceState.containsKey("isTaskWorking")) {
            isTaskWorking = savedInstanceState.getBoolean("isTaskWorking");
            if ( isTaskWorking ) {
                progressDialog.show();
            }
        }
    }

    //CICLOS ACTIVITY
    @Override
    public void onStart() {
        super.onStart();
        GoogleAnalytics.getInstance(this).reportActivityStart(this);
    }
    @Override
    protected void onRestart() {
        super.onRestart();
    }
    @Override
    public void onResume() {
        super.onResume();
        if (adView != null) {
            adView.resume();
        }
    }
    @Override
    public void onPause() {
        if (adView != null) {
            adView.pause();
        }
        super.onPause();
    }
    @Override
    public void onStop() {
        GoogleAnalytics.getInstance(this).reportActivityStop(this);
        super.onStop();
    }
    @Override
    public void onDestroy() {
        if (adView != null) {
            adView.destroy();
        }
        super.onDestroy();
    }
    //fin ciclos Activity
}
