package com.jesusrojo.miphoto.view;

import android.app.Activity;
import android.app.AlarmManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.os.Build;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.view.View;
import android.widget.CheckBox;
import android.widget.TextView;
import android.widget.TimePicker;

import java.util.Calendar;

import com.google.android.gms.ads.AdRequest;
import com.google.android.gms.ads.AdView;
import com.google.android.gms.analytics.GoogleAnalytics;
import com.jesusrojo.miphoto.R;
import com.jesusrojo.miphoto.presenter.broadcast.NotificationA_BroadcastReceiver;
import com.jesusrojo.miphoto.presenter.broadcast.NotificationRA_BroadcastReceiver;
import com.jesusrojo.miphoto.utils.Constants;
import com.jesusrojo.miphoto.utils.Utils;

public class ReminderActivity extends AppCompatActivity {

    protected final String TAG = getClass().getSimpleName() ;
    protected static Context context;
    protected Activity activity;
    private SharedPreferences.Editor editor;
    private CheckBox mCheckBox_2min, mCheckBox_hour,mCheckBox_day, mCheckBox_week;
    private TextView mTV_alarmTime,  mTV_title_checkbox;
    private TimePicker mTimePicker;

    private String alarmHour;
    private String alarmMinute;
    private long alarmTimeMillis;

    //intent for the repeating alarm
    private Intent mIntentRA;
    private PendingIntent mPendingIntentRA;
    
    //intent for the normal alarm
    //so if now is 17:05 and  we put the alarm at 17: 30, it will BUMM
    // at 17:30 and the it will use the RA intent every 24 hours
    //basically we not miss the first time after setting the alarm
    private Intent mIntentA;
    private PendingIntent mPendingIntentA;
    private AlarmManager mAlarmManager;
    private SharedPreferences mSharedPref;
    private final int MY_INTERVAL_2_MINUTES = 2 * 60 * 1000;//dos minutos
    private final int MY_INTERVAL_HOUR =       1 * 60 * 60 * 1000;
    private final int MY_INTERVAL_DAY =       24 * 60 * 60 * 1000;
    private final int MY_INTERVAL_WEEK =  7 * 24 * 60 * 60 * 1000;
    private  int interval_choosen ;
    private AdView adView;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.reminder_layout);
        Utils.showLog(TAG, "onCreate");
        context = getApplicationContext();
        activity = this;

        adView = (AdView)findViewById(R.id.adView);
        AdRequest adRequest = new AdRequest.Builder()
                .build();
        adView.loadAd(adRequest);
        //views
        mTV_alarmTime = (TextView)findViewById(R.id.TV_alarmTime);
        mTimePicker = (TimePicker)findViewById(R.id.timePicker);
        mTimePicker.setOnTimeChangedListener(new TimePicker.OnTimeChangedListener() {
            public void onTimeChanged(TimePicker view, int hourOfDay, int minute) {
                Utils.showLog(TAG, "onTimeChanged(" + hourOfDay + " " + minute);
            }
        });
        mTV_title_checkbox = (TextView)findViewById(R.id.TV_title_checkbox);
        mCheckBox_2min = (CheckBox) findViewById(R.id.CheckBoxReminder2min);
        mCheckBox_hour = (CheckBox) findViewById(R.id.CheckBoxReminderHour);
        mCheckBox_day = (CheckBox) findViewById(R.id.CheckBoxReminderDay);
        mCheckBox_week = (CheckBox) findViewById(R.id.CheckBoxReminderWeek);

        //iniciamos t0do a cero
        initAllViewsClean();

        //comprobamos alarma en shared preferences
         mSharedPref = getSharedPreferences(Constants.PREFERENCES_DEFAULT, Context.MODE_PRIVATE);
         alarmHour = mSharedPref.getString("hour", "");
         alarmMinute = mSharedPref.getString("minute", "");
         Utils.showLog(TAG, "Shared Preferences: " + alarmHour + " "+ alarmMinute);

        if( !alarmHour.equals("")  && !alarmMinute.equals("")) {
            //TENEMOS ALARMA
            String intervalo_pref = mSharedPref.getString("interval", "");
            Utils.showLog(TAG, "intervalo_pref: " + intervalo_pref);

            //comprobar el tipo de alarma
            if(intervalo_pref.equals("2min")){
                showPreferences2min();
            }
            if(intervalo_pref.equals("hour")){
                showPreferencesHour();
            }
            if(intervalo_pref.equals("day")){
                showPreferencesDay();
            }
            if(intervalo_pref.equals("week")){
                showPreferencesWeek();
            }
        }

    }//onCreate

    private void initAllViewsClean() {

        mTV_alarmTime.setBackgroundResource(0);
        mTV_alarmTime.setText(getResources().getString(R.string.notification_desactivated));

        mTV_title_checkbox.setText(getResources().getString(R.string.click_to_set_notification_every));

        mCheckBox_2min.setText(getResources().getString(R.string.minutes2));
        mCheckBox_2min.setChecked(false);
        mCheckBox_2min.setTextColor(Color.BLACK);
        
        mCheckBox_hour.setText(getResources().getString(R.string.hour));
        mCheckBox_hour.setChecked(false);
        mCheckBox_hour.setTextColor(Color.BLACK);

        mCheckBox_day.setText(getResources().getString(R.string.day));
        mCheckBox_day.setChecked(false);
        mCheckBox_day.setTextColor(Color.BLACK);

        mCheckBox_week.setText(getResources().getString(R.string.week));
        mCheckBox_week.setChecked(false);
        mCheckBox_week.setTextColor(Color.BLACK);
    }
    private void showPreferences2min() {
        mCheckBox_2min.setTextColor(Color.RED);
        mCheckBox_hour.setTextColor(Color.BLACK);
        mCheckBox_day.setTextColor(Color.BLACK);
        mCheckBox_week.setTextColor(Color.BLACK);
        mCheckBox_2min.setChecked(true);
        mCheckBox_hour.setChecked(false);
        mCheckBox_day.setChecked(false);
        mCheckBox_week.setChecked(false);
        String text =getResources().getString(R.string.notification_set_at)
                + " " + alarmHour + " : "+ alarmMinute + " "  + getString(R.string.every_2_min);
        mTV_alarmTime.setText(text);
        mTV_alarmTime.setBackgroundResource(R.color.verde_claro_android);
        Utils.showToast(context, text);
        mTV_title_checkbox.setText(getResources().getString(R.string.unclick_to_delete_alarm));
    }
    private void showPreferencesHour() {
        mCheckBox_2min.setTextColor(Color.BLACK);
        mCheckBox_hour.setTextColor(Color.RED);
        mCheckBox_day.setTextColor(Color.BLACK);
        mCheckBox_week.setTextColor(Color.BLACK);
        mCheckBox_2min.setChecked(false);
        mCheckBox_hour.setChecked(true);
        mCheckBox_day.setChecked(false);
        mCheckBox_week.setChecked(false);
        String text =getResources().getString(R.string.notification_set_at)
                + " " + alarmHour + " : "+ alarmMinute +" "  +  getString(R.string.every_hour);
        mTV_alarmTime.setText(text);
        mTV_alarmTime.setBackgroundResource(R.color.verde_claro_android);
        Utils.showToast(context, text);
        mTV_title_checkbox.setText(getResources().getString(R.string.unclick_to_delete_alarm));
    }

    private void showPreferencesDay() {
        mCheckBox_2min.setTextColor(Color.BLACK);
        mCheckBox_hour.setTextColor(Color.BLACK);
        mCheckBox_day.setTextColor(Color.RED);
        mCheckBox_week.setTextColor(Color.BLACK);
        mCheckBox_2min.setChecked(false);
        mCheckBox_hour.setChecked(false);
        mCheckBox_day.setChecked(true);
        mCheckBox_week.setChecked(false);
        String text =getResources().getString(R.string.notification_set_at)
                + " " + alarmHour + " : "+ alarmMinute +" "  +  getString(R.string.every_day);
        mTV_alarmTime.setText(text);
        mTV_alarmTime.setBackgroundResource(R.color.verde_claro_android);
        Utils.showToast(context, text);
        mTV_title_checkbox.setText(getResources().getString(R.string.unclick_to_delete_alarm));
    }

    private void showPreferencesWeek() {
        mCheckBox_2min.setTextColor(Color.BLACK);
        mCheckBox_day.setTextColor(Color.BLACK);
        mCheckBox_hour.setTextColor(Color.BLACK);
        mCheckBox_week.setTextColor(Color.RED);
        mCheckBox_2min.setChecked(false);
        mCheckBox_hour.setChecked(false);
        mCheckBox_day.setChecked(false);
        mCheckBox_week.setChecked(true);
        String text =getResources().getString(R.string.notification_set_at)
                + " " + alarmHour + " : "+ alarmMinute +" "  +  getString(R.string.every_week);
        mTV_alarmTime.setText(text);
        mTV_alarmTime.setBackgroundResource(R.color.verde_claro_android);
        Utils.showToast(context, text);
        mTV_title_checkbox.setText(getResources().getString(R.string.unclick_to_delete_alarm));
    }

    //CHECK BOX
    public void onCheckboxClicked(View view) {
        boolean checked = ((CheckBox) view).isChecked();
        switch (view.getId()) {
            case R.id.CheckBoxReminder2min:
                if (checked) {

                    setTimeForAlarm();
                    showPreferences2min();

                    editor = mSharedPref.edit();
                    editor.putString("interval", "2min");
                    editor.apply();
                    createNotificationA(alarmTimeMillis);
                    createNotificationRA(alarmTimeMillis, MY_INTERVAL_2_MINUTES);
                    interval_choosen = MY_INTERVAL_2_MINUTES;
                } else {
                    cancelEverything();
                    mCheckBox_2min.setTextColor(Color.BLACK);
                }
                break;
            case R.id.CheckBoxReminderHour:
                if (checked) {

                    setTimeForAlarm();
                    showPreferencesHour();

                    editor = mSharedPref.edit();
                    editor.putString("interval", "hour");
                    editor.apply();
                    createNotificationA(alarmTimeMillis);
                    createNotificationRA(alarmTimeMillis, MY_INTERVAL_HOUR);
                    interval_choosen = MY_INTERVAL_HOUR;
                } else {
                    cancelEverything();
                    mCheckBox_hour.setTextColor(Color.BLACK);
                }
                break;
            case R.id.CheckBoxReminderDay:
                if (checked) {

                    setTimeForAlarm();
                    showPreferencesDay();

                    editor = mSharedPref.edit();
                    editor.putString("interval", "day");
                    editor.apply();
                    createNotificationA(alarmTimeMillis);
                    createNotificationRA(alarmTimeMillis, MY_INTERVAL_DAY);
                    interval_choosen = MY_INTERVAL_DAY;
                } else {
                    cancelEverything();
                    mCheckBox_day.setTextColor(Color.BLACK);
                }
            break;
            case R.id.CheckBoxReminderWeek:
                if (checked) {
                    setTimeForAlarm();
                    showPreferencesWeek();

                    editor = mSharedPref.edit();
                    editor.putString("interval", "week");
                    editor.apply();
                    createNotificationA(alarmTimeMillis);
                    createNotificationRA(alarmTimeMillis, MY_INTERVAL_WEEK);
                    interval_choosen = MY_INTERVAL_WEEK;
                } else {
                    cancelEverything();
                    mCheckBox_week.setTextColor(Color.BLACK);
              }
               break;
        }
    }

    private void cancelEverything() {
        cancelNotificationA();
        cancelNotificationRA();
        editor = mSharedPref.edit();
        editor.putString("2min", "");
        editor.putString("hour", "");
        editor.putString("minute", "");
        editor.putString("interval", "");
        editor.apply();

        mTV_alarmTime.setBackgroundResource(R.color.gris_55);
        mTV_alarmTime.setText(R.string.notification_desactivated);

        mTV_title_checkbox.setText(getResources().getString(R.string.click_to_set_notification_every));
        Utils.showToast(context, getResources().getString(R.string.notification_canceled));

    }

    private void setTimeForAlarm() {
        //GET time from picker
        int  hour;
        int minute;
        if (  Build.VERSION.SDK_INT < 23 ) {
            hour = mTimePicker.getCurrentHour();
            minute = mTimePicker.getCurrentMinute();
        } else {
            hour = mTimePicker.getHour();
            minute = mTimePicker.getMinute();
        }
        alarmHour= String.valueOf(hour);
        alarmMinute= String.valueOf(minute);
        Utils.showLog(TAG, "time from picker: " + alarmHour + " " + alarmMinute);

        Calendar calendarAlarm = Calendar.getInstance();
        calendarAlarm.set(Calendar.HOUR_OF_DAY,hour);
        calendarAlarm.set(Calendar.MINUTE, minute);
        calendarAlarm.set(Calendar.SECOND, 0);

        //SET alarm tim
        alarmTimeMillis =  calendarAlarm.getTimeInMillis();
        Utils.showLog(TAG, "alarmTimeMillis " + alarmTimeMillis);
        Utils.showLog(TAG, "System.currentTimeMillis() " + System.currentTimeMillis());
        //if we make the alarm before the actual time
        //and dont want to see the notification once saved
        if(alarmTimeMillis < System.currentTimeMillis()) {
           // alarmTimeMillis = alarmTimeMillis + AlarmManager.INTERVAL_DAY;
            alarmTimeMillis = alarmTimeMillis + interval_choosen;
        }

        //put time in View
        String resp;
        //si es menor de 10 annadimos un cero a la izquierda
        if(minute<10){
            alarmMinute= "0" + alarmMinute;
        }
        resp = getResources().getString(R.string.notification_set_at)
                + " " +alarmHour + " : " + alarmMinute;
        mTV_alarmTime.setText(resp);
        mTV_alarmTime.setBackgroundResource(R.color.verde_claro_android);

        //save time in preferences
        editor = mSharedPref.edit();
        editor.putString("hour", alarmHour);
        editor.putString("minute", alarmMinute);
        editor.apply();
    }

    //RA
    private void createNotificationRA(long time, int interval) {
        mIntentRA = new Intent(this, NotificationRA_BroadcastReceiver.class);
        mPendingIntentRA = PendingIntent.getBroadcast(this, 0, mIntentRA, 0);

        mAlarmManager =(AlarmManager) getSystemService(Context.ALARM_SERVICE);
        mAlarmManager.setRepeating(AlarmManager.RTC_WAKEUP,
                time + interval,
                interval,
                mPendingIntentRA);
    }
    private void cancelNotificationRA() {
        mIntentRA = new Intent(this, NotificationRA_BroadcastReceiver.class);
        mPendingIntentRA = PendingIntent.getBroadcast(this, 0, mIntentRA, 0);

        mAlarmManager= (AlarmManager) getSystemService(ALARM_SERVICE);
        mAlarmManager.cancel(mPendingIntentRA);
    }
    //A
    private void createNotificationA(long time) {
        mIntentA = new Intent(this, NotificationA_BroadcastReceiver.class);
        mPendingIntentA = PendingIntent.getBroadcast(this, 0, mIntentA, 0);
        mAlarmManager = (AlarmManager) this.getSystemService(ALARM_SERVICE);
        mAlarmManager.set(AlarmManager.RTC_WAKEUP,
                alarmTimeMillis,
                mPendingIntentA);
    }
    private void cancelNotificationA() {
        mIntentA = new Intent(this, NotificationA_BroadcastReceiver.class);
        mPendingIntentA = PendingIntent.getBroadcast(this, 0, mIntentA, 0);

        mAlarmManager= (AlarmManager) getSystemService(ALARM_SERVICE);
        mAlarmManager.cancel(mPendingIntentA);
    }

    public void salir(View view) {
        finish();
    }
    //CICLOS ACTIVITY
    @Override
    public void onStart() {
        super.onStart();
        GoogleAnalytics.getInstance(this).reportActivityStart(this);
    }
    @Override
    protected void onRestart() {
        super.onRestart();
    }
    @Override
    public void onResume() {
        super.onResume();
        if (adView != null) {
            adView.resume();
        }
    }
    @Override
    public void onPause() {
        if (adView != null) {
            adView.pause();
        }
        super.onPause();
    }
    @Override
    public void onStop() {
        GoogleAnalytics.getInstance(this).reportActivityStop(this);
        super.onStop();
    }
    @Override
    public void onDestroy() {
        if (adView != null) {
            adView.destroy();
        }
        super.onDestroy();
    }
    //fin ciclos Activity

}



