package com.jesusrojo.miphoto.view.ui;


import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Build;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.ToggleButton;

import com.jesusrojo.miphoto.R;
import com.jesusrojo.miphoto.model.pojo.Photo;
import com.jesusrojo.miphoto.utils.Utils;
import com.jesusrojo.miphoto.utils.UtilsFile;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

public class PhotosBaseAdapter extends BaseAdapter {

    private final String TAG = getClass().getSimpleName();

    //true means  the path is selected, false means  pathFilter is selected
    public static boolean isPath;

    //  DATA
    public  List<Photo> listPhotos = new ArrayList<Photo>();
    private boolean[] checked;

    //holder
    static class ViewHolder {
        ImageView mIV_image;
        TextView mTV_id;
        ImageView mIV_status;
        TextView mTV_uri;
        TextView mTV_owner;
        TextView mTV_time;
        TextView mTV_comment;
        CheckBox mCheckBox;
        ToggleButton toggleButton;
    }

    private Context mContext;
    private LayoutInflater mInflater;

    public PhotosBaseAdapter(Context c) {
        super();
        this.mContext = c;
        mInflater = LayoutInflater.from(c);
        isPath = true;
    }
    @Override
    public View getView(final int position, View convertView, ViewGroup parent) {
        //holder
        ViewHolder holder;

        if (convertView == null) {
            convertView =
                    mInflater.inflate(R.layout.photos_list_item_layout,null);
            //create holder
            holder = new ViewHolder();

            //add to holder
            holder.mIV_image = (ImageView) convertView.findViewById(R.id.IV_photo_icon);
            holder.mTV_id =(TextView) convertView.findViewById(R.id.TV_id);
            holder.mIV_status =(ImageView) convertView.findViewById(R.id.IV_status);
            holder.mTV_uri =(TextView) convertView.findViewById(R.id.TV_uri);
            holder.mTV_owner =(TextView) convertView.findViewById(R.id.TV_owner);
            holder.mTV_comment =(TextView) convertView.findViewById(R.id.TV_comment);
            holder.mTV_time =(TextView) convertView.findViewById(R.id.TV_time);
            holder.mCheckBox = (CheckBox) convertView.findViewById(R.id.checkBox);
            holder.toggleButton = (ToggleButton) convertView.findViewById(R.id.toggleButton);

            //set tag holder
            convertView.setTag(holder);
        }
            //get  tag holder
            holder = (ViewHolder) convertView.getTag();

        //DATA======================================
        final Photo photo = listPhotos.get(position);
        //comprobamos si existe el archivo binario
        // y el nombre del path en SQLite
        File f = new File(String.valueOf(photo.getPath().substring(8)));
            if (f.getAbsoluteFile().exists()
                  && !TextUtils.isEmpty(photo.getPath())) {
                holder.mIV_image.setImageBitmap(
                        UtilsFile.reduceBitmap(mContext, photo.getPath(), 500, 500));
                holder.mIV_image.setScaleType(ImageView.ScaleType.FIT_CENTER);
                holder.mIV_image.setOnClickListener(
                    new View.OnClickListener() {
                        @Override
                        public void onClick(View v) {
                            /////OPEN FULL VIEW ORIGINAL
                            openFullViewOriginal(position);
                        }
                    }
                );
            } else {    //put icon error
               if (  Build.VERSION.SDK_INT < 21 ) {
                   holder.mIV_image.setImageDrawable(mContext.getResources().getDrawable(android.R.drawable.ic_menu_report_image));
               } else {
                   holder.mIV_image.setImageDrawable(mContext.getResources().getDrawable(android.R.drawable.ic_menu_report_image,null));
               }
             //   Utils.showToast(mContext, mContext.getResources().getString(R.string.error_files_not_exits));
            }
        holder.mTV_id.setText(String.valueOf(photo.getId()));

        int status = photo.getStatus();
        if(status == 1 ) {
            if (  Build.VERSION.SDK_INT < 21 ) {
                holder.mIV_status.setImageDrawable(mContext.getResources().getDrawable(R.drawable.nube_done));
            } else {
                holder.mIV_status.setImageDrawable(mContext.getResources().getDrawable(R.drawable.nube_done,null));
            }
           // holder.mTV_status.setText("C");
        } else {
           // holder.mTV_status.setText("");
        }
        
        holder.mTV_uri.setText(photo.getPath() );
        if(!TextUtils.isEmpty(photo.getComment())) {
            holder.mTV_comment.setVisibility(View.VISIBLE);
            holder.mTV_comment.setText(photo.getComment());
        } else {
            holder.mTV_comment.setVisibility(View.GONE);
        }
        holder.mTV_owner.setText(photo.getOwner() );
        try {
            holder.mTV_time.setText(Utils.millisToHuman(photo.getTime()));
        } catch (NumberFormatException nfe) {
            Utils.showLogError(TAG, "NumberFormatException: " + nfe.getMessage());
        }
        //CHECK BOX
        /*1 abajo*/
       // holder.mCheckBox.setOnClickListener(myClickListener);
        /*2*/
        //Important to remove previous listener before calling setChecked
        holder.mCheckBox.setOnCheckedChangeListener(null);
        holder.mCheckBox.setTag(position);
        holder.mCheckBox.setChecked(checked[position]);
        holder.mCheckBox.setOnCheckedChangeListener(
                new CompoundButton.OnCheckedChangeListener() {
                    @Override
                    public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {

                        // mItemChecked[posicion] = isChecked;
                         checked[(Integer)buttonView.getTag()] = isChecked;
                         notifyDataSetChanged();

                        Utils.showLog(TAG, "Checked: " +
                                checked[(Integer)buttonView.getTag()]
                                +" "+ buttonView.getTag());
                    }
                });
        ///
        /*******************************************/
        //TOGGLE
        final ViewHolder finalHolder = holder;
        holder.toggleButton.setChecked(false);
        holder.toggleButton.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
            if (isChecked) {
                isPath = false;
                // The toggle is enabled PATH FILTER
                if (photo.getPathFilter().equals("")) {
                    //put icon error
                  //  Utils.showToast(mContext, mContext.getResources().getString(R.string.there_is_not_filter));
                    if (  Build.VERSION.SDK_INT < 21 ) {
                        finalHolder.mIV_image.setImageDrawable(mContext.getResources().getDrawable(android.R.drawable.ic_menu_report_image));
                    } else {
                        finalHolder.mIV_image.setImageDrawable(mContext.getResources().getDrawable(android.R.drawable.ic_menu_report_image,null));}

                } else {    //put the pathFilter on view

                    finalHolder.mIV_image.setImageBitmap(
                            //  UtilsFile.getThumbnail(photo.getPath()) );
                            UtilsFile.reduceBitmap(mContext,  photo.getPathFilter(), 500, 500));
                    finalHolder.mIV_image.setScaleType(ImageView.ScaleType.FIT_CENTER);
                    finalHolder.mIV_image.setOnClickListener(
                            new View.OnClickListener() {
                                @Override
                                public void onClick(View v) {
                                     ///////OPEN FULL VIEW FILTER
                                    openFullViewFilter(position);
                                }
                            }
                    );
                }
            } else {    // The toggle is disabled    PATH
               // Utils.showLog(TAG, "photo.getPath(): " + photo.getPath());
                isPath = true;
                 if (!TextUtils.isEmpty(photo.getPath())) {
                    finalHolder.mIV_image.setImageBitmap(
                            //  UtilsFile.getThumbnail(photo.getPath()) );
                            UtilsFile.reduceBitmap(mContext, photo.getPath(), 500, 500));
                    finalHolder.mIV_image.setScaleType(ImageView.ScaleType.FIT_CENTER);
                    finalHolder.mIV_image.setOnClickListener(
                            new View.OnClickListener() {
                                @Override
                                public void onClick(View v) {
                                     //////OPEN FULL VIEW ORIGINAL
                                    openFullViewOriginal(position);
                                }
                            }
                    );
                } else { //put icon error
                   if (  Build.VERSION.SDK_INT < 21 ) {
                       finalHolder.mIV_image.setImageDrawable(mContext.getResources().getDrawable(android.R.drawable.ic_menu_report_image));
                   } else {
                      finalHolder.mIV_image.setImageDrawable(mContext.getResources().getDrawable(android.R.drawable.ic_menu_report_image,null));}
                 }
            }
            }
        });
        /****************************/
        //end DATA

        return convertView;
    }

    private void openFullViewOriginal(int position) {
        Intent fullSelfieIntent = new Intent(Intent.ACTION_VIEW);
        fullSelfieIntent.setDataAndType(
                Uri.parse(listPhotos.get(position).getPath()),
                "image/*");
        mContext.startActivity(fullSelfieIntent);
        ///
     /*back to  PHOTO ACTIVITY on Click in the image **********
        Intent intent = new Intent(mContext, PhotoSaveEditActivity.class);
        intent.putExtra("path", listPhotos.get(position).getPath());
        mContext.startActivity(intent);
        // finish from base adapter
        ((Activity) mContext).finish();
        ///
    ********/
    }
    private void openFullViewFilter(int position) {
        Intent fullSelfieIntent = new Intent(Intent.ACTION_VIEW);
        fullSelfieIntent.setDataAndType(
                Uri.parse(listPhotos.get(position).getPathFilter()),
                "image/*");
        mContext.startActivity(fullSelfieIntent);
    }

    public void setListaSelfies(List<Photo> lista) {
        this.listPhotos = lista;

        checked= new boolean[listPhotos.size()];
        for(int i=0; i < checked.length; i++){
            checked[i]=false;
        }
        notifyDataSetChanged();
        Utils.showLog(TAG, "Lista actualizada");
      }

    public List<Photo> getListPhotos() {
        return listPhotos;
    }

    //CHECKED checkbox staff
    public List<Photo> getListPhotosSelected(){
       List<Photo> listCheckedPhotos = new ArrayList<>();
       //create a list with the selected from the arrayCheked
       for(int i=0; i < checked.length; i++){
           if ( checked[i] ) {
                listCheckedPhotos.add(listPhotos.get(i));
           }
       }
       return listCheckedPhotos;
   }

    public void checkAll(boolean isChecked){
        for(int i=0; i < checked.length; i++){
            checked[i]=isChecked;
        }
        notifyDataSetChanged();
    }
    public boolean[] getAllSelected() {
       return checked;
    }
    public boolean itemIsChecked(int i){
        return checked[i];
    }
    ///

    @Override
    public int getCount() {
        return listPhotos.size();
    }
    @Override
    public Object getItem(int position) {
        return listPhotos.get(position);
    }
    @Override
    public long getItemId(int position) {
        return position;
    }

    /*****************************1CHECK BOX OnCliclik
    private View.OnClickListener myClickListener = new View.OnClickListener() {
        @Override
        public void onClick(View v) {

             int posicion= (int) v.getTag();
             boolean checked = ((CheckBox) v).isChecked();

             if (checked) {
             Utils.showToast(mContext, "checked: " + posicion);
             Utils.showLog(TAG, "checked: " + posicion);
             //  checked[(Integer)buttonView.getTag()] = isChecked;
             // listPhotos.get(posicion).setChecked(true);
             } else {
             Utils.showToast(mContext, "un-checked: " + posicion);
             Utils.showLog(TAG, "un-checked: " + posicion);
             // listPhotos.get(posicion).setChecked(false);
             //showLogListPhotos();
             }
        }
    };//end************************/
}
